//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "EPRound.h"
#include "playerlist.h"


EPRound::EPRound(int round, unsigned int player)
    :ERound(round), d_player(player)
{
    d_type = PROUND;
}

EPRound::EPRound(XML_Helper* helper)
    :ERound(helper)
{
    helper->getData(d_player, "player");
    d_type = PROUND;
}

EPRound::~EPRound()
{
}

bool EPRound::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("event");
    retval &= helper->saveData("type", std::string("EPROUND"));
    retval &= helper->saveData("round", d_round);
    retval &= helper->saveData("player", d_player);

    retval &= Event::save(helper);

    retval &= helper->closeTag();
    
    return retval;
}

void EPRound::trigger(Player* p)
{
    if (!d_active || (sgettingRound.emit() != d_round) || (p->getId() != d_player))
        return;

    d_active = false;
    raise();
}

Player* EPRound::getPlayer()
{
    Playerlist* plist = Playerlist::getInstance();
    for (Playerlist::iterator it = plist->begin(); it != plist->end(); it++)
        if ((*it)->getId() == d_player)
            return (*it);

    return 0;
}
