//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "Event.h"
#include "EPlayerDead.h"
#include "EKillAll.h"
#include "ECityConq.h"
#include "EPCityConq.h"
#include "EArmyKilled.h"
#include "ERound.h"
#include "EPRound.h"
#include "ERuinSearch.h"


Event::Event(TYPE type)
    :d_type(type), d_active(true)
{
}

Event::Event(XML_Helper* helper)
{
    helper->getData(d_active, "active");

    helper->registerTag("reaction", SigC::slot((*this), &Event::loadReaction));
}

Event::~Event()
{
    while (!d_reactions.empty())
    {
        delete (*d_reactions.begin());
        d_reactions.erase(d_reactions.begin());
    }
}

Event* Event::loadEvent(XML_Helper* helper)
{
    std::string typestring;
    helper->getData(typestring, "type");

    if (typestring == std::string("EPLAYERDEAD"))
        return new EPlayerDead(helper);
    if (typestring == std::string("EKILLALL"))
        return new EKillAll(helper);
    if (typestring == std::string("ECITYCONQUERED"))
        return new ECityConq(helper);
    if (typestring == std::string("EPCITYCONQUERED"))
        return new EPCityConq(helper);
    if (typestring == std::string("EARMYKILLED"))
        return new EArmyKilled(helper);
    if (typestring == std::string("EROUND"))
        return new ERound(helper);
    if (typestring == std::string("EPROUND"))
        return new EPRound(helper);
    if (typestring == std::string("ERUINSEARCH"))
        return new ERuinSearch(helper);
    
    return 0;
}

bool Event::loadReaction(std::string tag, XML_Helper* helper)
{
    if (tag != "reaction")
        return false;

    d_reactions.push_back(Reaction::loadReaction(helper));
    return true;
}

bool Event::save(XML_Helper* helper)
{
    //Note that this function doesn't open or close tags, this has to be done
    //by the derived classes!
    bool retval = true;

    retval &= helper->saveData("active", d_active);
    
    //save the reactions
    std::list<Reaction*>::iterator it;
    for (it = d_reactions.begin(); it != d_reactions.end(); it++)
        retval &= (*it)->save(helper);

    return retval;
}

void Event::addReaction(Reaction* r)
{
    d_reactions.push_back(r);
}

Reaction* Event::getReaction(int index)
{
    std::list<Reaction*>::iterator it = d_reactions.begin();

    while (index > 0)
    {
        it++;
        if (it == d_reactions.end())
            return 0;
        index--;
    }
    
    return *it;
}

void Event::raise()
{
    std::list<Reaction*>::iterator it;
    for (it = d_reactions.begin(); it != d_reactions.end(); it++)
        (*it)->trigger();
}
