//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <pgapplication.h>
#include <stdio.h>
#include "ItemDialog.h"
#include "GameMap.h"
#include "defs.h"
//for getting the position
#include "player.h"
#include "stacklist.h"
#include "stack.h"
#include "File.h"

ItemDialog::ItemDialog(Hero* hero, PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, hero->getName().c_str(), WF_MODAL),
    d_hero(hero), d_ground(0), d_back(0), d_active(0)
{
    // Before we start, we need the position of the hero. Getting it as parameter
    // would either mean reaching it through two other classes or putting
    // the code elsewhere, so one can as well get the position here.
    Stacklist* slist = hero->getOwner()->getStacklist();
    for (Stacklist::iterator it = slist->begin(); it != slist->end(); it++)
        for (Stack::iterator sit = (*it)->begin(); sit != (*it)->end(); sit++)
            if ((*sit) ==  hero)
                d_pos = (*it)->getPos();
    
    
    // This is neccessary due to a bug in paragui which would have problems with
    // resetting the icon to nothing
    d_tempsurf = File::getItemPicture("nobutton");
    
    // First, initialise the whole stuff I assume that an item has a size
    // of 40x40 pixels. From a short calculation of my layout, I get a
    // neccessary size of ~450x400 with enough free space

    // in the top left corner, place the image of the hero with his equiped
    // items around.
    d_b_pic = new PG_Button(this, 1, PG_Rect(100, 110, 50, 50), "");
    d_b_pic->EnableReceiver(false);
    d_b_pic->SetIcon(d_hero->getPixmap(), 0, 0);

    d_l_weapon = new PG_Label(this, PG_Rect(20, 95, 70, 20), _("weapon"));
    d_b_weapon = new PG_Button(this, 2, PG_Rect(30, 115, 40, 40), "");
    d_b_weapon->SetToggle(true);

    d_l_armour = new PG_Label(this, PG_Rect(105, 30, 70, 20), _("armour"));
    d_b_armour = new PG_Button(this, 3, PG_Rect(105, 50, 40, 40), "");
    d_b_armour->SetToggle(true);

    d_l_shield = new PG_Label(this, PG_Rect(180, 95, 70, 20), _("shield"));
    d_b_shield = new PG_Button(this, 4, PG_Rect(180, 115, 40, 40), "");
    d_b_shield->SetToggle(true);

    d_l_accessoire = new PG_Label(this, PG_Rect(95, 175, 80, 20), _("accessoire"));
    d_b_accessoire = new PG_Button(this, 5, PG_Rect(105, 195, 40, 40), "");
    d_b_accessoire->SetToggle(true);

    // On the right side, display the backpack.
    d_l_back = new PG_Label(this, PG_Rect(350, 40, 80, 20), _("Backpack"));
    d_b_backprev = new PG_Button(this, 6, PG_Rect(370, 70, 40, 20), _("prev"));
    for (int i = 0; i < 3; i++)
    {
        d_b_back[i] = new PG_Button(this, 7+i, PG_Rect(370, 100+i*50, 40, 40), "");
        d_b_back[i]->SetToggle(true);
    }
    d_b_backnext = new PG_Button(this, 11, PG_Rect(370, 250, 40, 20), _("next"));

    // On the bottom, display the items on the ground.
    d_l_ground = new PG_Label(this, PG_Rect(20, 260, 80, 20), _("Ground"));
    d_b_groundprev = new PG_Button(this, 12, PG_Rect(20, 310, 40, 20), _("prev"));
    for (int i = 0; i < 3; i++)
    {
        d_b_ground[i] = new PG_Button(this, 13+i, PG_Rect(80 + i*50, 300, 40, 40), "");
        d_b_ground[i]->SetToggle(true);
    }
    d_b_groundnext = new PG_Button(this, 16, PG_Rect(240, 310, 40, 20), _("next"));
    
    // A button at the bottom of the screen would be useful to quit the dialog :)
    d_b_ok = new PG_Button(this, 30, PG_Rect(370, 310, 50, 30), "OK");

    // And don't forget the labels for the item description
    d_l_item = new PG_Label(this, PG_Rect(200, 180, 150, 20), "");
    for (int i = 0; i < 3; i++)
        d_l_bonus[i] = new PG_Label(this, PG_Rect(200, 200+i*20, 150, 20), "");
    
    // Now connect all the event slots
    // First, the OK button which quits the dialog
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::okClicked);

    // Then the scroll buttons
    d_b_backprev->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::backScroll);
    d_b_backnext->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::backScroll);
    d_b_groundprev->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::groundScroll);
    d_b_groundnext->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::groundScroll);

    // Finally, all the item buttons
    d_b_weapon->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
    d_b_shield->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
    d_b_armour->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
    d_b_accessoire->SetEventObject(MSG_BUTTONCLICK, this,
                    (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
    for (int i = 0; i < 3; i++)
    {
        d_b_ground[i]->SetEventObject(MSG_BUTTONCLICK, this,
                       (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
        d_b_back[i]->SetEventObject(MSG_BUTTONCLICK, this,
                     (MSG_CALLBACK_OBJ)&ItemDialog::itemClick);
    }

    //And, finally, draw all the items.
    drawEquipment();
    drawBackpack();
    drawGround();
}

ItemDialog::~ItemDialog()
{
    if (d_tempsurf)
        SDL_FreeSurface(d_tempsurf);
}

PARAGUI_CALLBACK(ItemDialog::itemClick)
{
    PG_Button* select = static_cast<PG_Button*>(widget);
    
    // If no item has been selected yet, select the given item
    if (!d_active)
    {
        d_active = select;
        if (displayItem(getItem(d_active))!=0)
            return true;
	else 
        {
            d_active->SetPressed(false);
            select->SetPressed(false);
            d_active = 0;
            return true;
        }
    }
    
    // If the item is deselected again, do nothing as well.
    if (d_active == select)
    {
        d_active = 0;
        return true;
    }

    // Now the procedure is quite straightforward. We take the one item and
    // insert it at the position of the other slot, then we do it the other
    // way round. A single problem: If one of the slots is an equipment
    // slot, there is the possibility of a failure because the item type
    // doesn't fit.
    if (!isEquipment(d_active) && !isEquipment(select))
    {
        // straightforward: exchange the two
        // ATTENTION: First get the items to make sure you get the correct ones!
        Item* iselect = getItem(select, true);
        Item* iactive = getItem(d_active, true);
        insertItem(iactive, select);
        insertItem(iselect, d_active);

        // redraw and undo the whole stuff
        d_active->SetPressed(false);
        select->SetPressed(false);

        // we want to display the correct item's information
	displayItem(getItem(d_active));

        drawBackpack();
        drawGround();
        d_active = 0;
        
        return true;
    }

    // one of the buttons is the equipment button; set it to d_active
    if (!isEquipment(d_active))
    {
        PG_Button* temp = select;
        select = d_active;
        d_active = temp;
    }

    // Now save the equipment item and remove it, first
    Item* save = getItem(d_active, true);

    // Try to equip the other item. If it works, exchange these two, else put
    // the original item back on.
    if (insertItem(getItem(select, false), d_active))
    {
        // putting item on worked
        getItem(select, true);
        insertItem(save, select);
        // we want to display the correct item's information
	displayItem(getItem(d_active));
    }
    else
    {
        // putting on item did not work, insert saved item to equipment again
        insertItem(save, d_active);
    }

    // redraw
    d_active->SetPressed(false);
    select->SetPressed(false);
    d_active = 0;
    drawBackpack();
    drawGround();
    drawEquipment();
    
    return true;
}

PARAGUI_CALLBACK(ItemDialog::groundScroll)
{
    // Get the ground equipment list
    std::list<Item*> ground = GameMap::getInstance()->getTile(d_pos.x, d_pos.y)
                              ->getItems();

    // If it is possible to scroll forward, do it
    if ((widget == d_b_groundnext) && (d_ground + 3 <= ground.size()))
        d_ground++;

    // the same backwards
    if ((widget == d_b_groundprev) && (d_ground > 0))
        d_ground--;

    if (d_active)
    {
        d_active->SetPressed(false);
        d_active = 0;
    }
    
    // Redraw everything; the drawing functions also care about enabling/disabling the 
    // scroll buttons.
    drawGround();
    
    return true;
}

PARAGUI_CALLBACK(ItemDialog::backScroll)
{
    // Basically the same as with the ground, so I won't comment on it.
    std::list<Item*> back = d_hero->getBackpack();

    if ((widget == d_b_backnext) && (d_back + 3 <= back.size()))
        d_back++;

    if ((widget == d_b_backprev) && (d_back > 0))
        d_back--;
    
    if (d_active)
    {
        d_active->SetPressed(false);
        d_active = 0;
    }

    drawBackpack();
    
    return true;
}

PARAGUI_CALLBACK(ItemDialog::okClicked)
{
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

void ItemDialog::drawEquipment()
{
    // Deactivate the paragui updates, so everything is drawn in one rush
    PG_Application::SetBulkMode(true);

    // Clean all slots...
    d_b_weapon->SetIcon(d_tempsurf, 0, 0);
    d_b_shield->SetIcon(d_tempsurf, 0, 0);
    d_b_armour->SetIcon(d_tempsurf, 0, 0);
    d_b_accessoire->SetIcon(d_tempsurf, 0, 0);

    // ...and refill them with the actual values
    std::list<Item*> equip = d_hero->getEquipment();
    for (std::list<Item*>::iterator it = equip.begin(); it != equip.end(); it++)
    {
        Item* item = (*it);
        switch(item->getType())
        {
            case Item::WEAPON:
                d_b_weapon->SetIcon(item->getPic(), 0, 0);
                break;
            case Item::SHIELD:
                d_b_shield->SetIcon(item->getPic(), 0, 0);
                break;
            case Item::ARMOUR:
                d_b_armour->SetIcon(item->getPic(), 0, 0);
                break;
            case Item::ACCESSOIRE:
                d_b_accessoire->SetIcon(item->getPic(), 0, 0);
                break;
            case Item::NONE:
                break;
        }
    }

    // Update the changes
    PG_Application::SetBulkMode(false);
    Update();
}

void ItemDialog::drawBackpack()
{
    // First, deactivate paragui updates again
    PG_Application::SetBulkMode(true);

    // Get the backpack and set the iterator to the right position
    std::list<Item*> pack = d_hero->getBackpack();
    
    std::list<Item*>::iterator it = pack.begin();
    for (unsigned int j = 0; j < d_back; j++, it++);

    // And set each button's icons
    for (int i = 0; i < 3; i++)
    {
        if (it == pack.end())
        {
            d_b_back[i]->SetIcon(d_tempsurf, 0, 0);
            continue;
        }
        
        d_b_back[i]->SetIcon((*it)->getPic(), 0, 0);
        it++;
    }

    // Finally, enable/disable the scrolling buttons. We allow one empty slot
    // so the player can actually put things in the backpack instead of
    // swapping items.

    d_b_backnext->EnableReceiver(false);
    if (d_back + 3 <= pack.size())
        d_b_backnext->EnableReceiver(true);

    d_b_backprev->EnableReceiver(false);
    if (d_back > 0)
        d_b_backprev->EnableReceiver(true);
    
    // Update the changes
    PG_Application::SetBulkMode(false);
    Update();
}

void ItemDialog::drawGround()
{
    // Almost the same as in drawBackpack

    // Deactivate paragui updates
    PG_Application::SetBulkMode(true);

    // Get the list of items on the ground.
    std::list<Item*> ground = GameMap::getInstance()->getTile(d_pos.x, d_pos.y)
                              ->getItems();

    std::list<Item*>::iterator it = ground.begin();
    for (unsigned int j = 0;j < d_ground; j++, it++);

    //Set each button's icons
    for (int i = 0; i < 3;i++)
    {
        if (it == ground.end())
        {
            d_b_ground[i]->SetIcon(d_tempsurf, 0, 0);
            continue;
        }

        d_b_ground[i]->SetIcon((*it)->getPic(), 0, 0);
        it++;
    }

    // Check the scrolling buttons
    d_b_groundnext->EnableReceiver(false);
    if (d_ground +3 <= ground.size())
        d_b_groundnext->EnableReceiver(true);

    d_b_groundprev->EnableReceiver(false);
    if (d_ground > 0)
        d_b_groundprev->EnableReceiver(true);
    
    //Update the changes
    PG_Application::SetBulkMode(false);
    Update();
}

Item* ItemDialog::getItem(PG_Button* button, bool erase)
{
    Item* retval = 0;
    
    // handling of equipped items
    if (isEquipment(button))
    {
        Item::TYPE itemtype=Item::NONE;

        if (button == d_b_weapon)
            itemtype = Item::WEAPON;
        if (button == d_b_shield)
            itemtype = Item::SHIELD;
        if (button == d_b_armour)
            itemtype = Item::ARMOUR;
        if (button == d_b_accessoire)
            itemtype = Item::ACCESSOIRE;

        std::list<Item*> ilist = d_hero->getEquipment();
        for (std::list<Item*>::iterator it = ilist.begin(); it != ilist.end(); it++)
            if (itemtype == (*it)->getType())
            {
                retval = (*it);
                if (erase)
                    d_hero->removeFromEquipment(*it);
                return retval;
            }

        return 0;
    }

    // handle items in the backpack
    for (int i = 0; i < 3; i++)
    {
        if (button == d_b_back[i])
        {
            unsigned int itempos = d_back + i;
            std::list<Item*> ilist = d_hero->getBackpack();
            std::list<Item*>::iterator it = ilist.begin();
            
            if (itempos >= ilist.size())
                return 0;
            
            for (unsigned int i = 0; i < itempos; it++, i++);
            retval = (*it);

            if (erase)
                d_hero->removeFromBackpack(*it);
            return retval;
        }
    }

    // handle items on the ground
    for (int i = 0; i < 3; i++)
    {
        if (button == d_b_ground[i])
        {
            unsigned int itempos = d_ground + i;
            std::list<Item*> ilist = GameMap::getInstance()
                                     ->getTile(d_pos.x, d_pos.y)->getItems();
            std::list<Item*>::iterator it = ilist.begin();
            
            if (itempos >= ilist.size())
                return 0;
            
            for (unsigned int i = 0; i < itempos; it++, i++);
            retval = (*it);

            if (erase)
                GameMap::getInstance()->getTile(d_pos.x, d_pos.y)->removeItem(*it);
            return retval;
        }
    }

    return 0;
}

bool ItemDialog::isEquipment(PG_Button* button)
{
    if ((button == d_b_weapon)
        || (button == d_b_shield)
        || (button == d_b_armour)
        || (button == d_b_accessoire))
        return true;

    return false;
}

bool ItemDialog::insertItem(Item* item, PG_Button* pos)
{
    if (!item)
        return true;

    if (isEquipment(pos))
    {
        Item::TYPE itemtype=Item::NONE;

        if (pos == d_b_weapon)
            itemtype = Item::WEAPON;
        if (pos == d_b_shield)
            itemtype = Item::SHIELD;
        if (pos == d_b_armour)
            itemtype = Item::ARMOUR;
        if (pos == d_b_accessoire)
            itemtype = Item::ACCESSOIRE;

        // first check: has the item the correct type
        if (item->getType() != itemtype)
            return false;

        // Insert the item into the equipment. The function returns false on error
        return d_hero->addToEquipment(item);
    }

    // no equipment, so let's find out where this item comes from
    for (int i = 0; i < 3; i++)
    {
        if (pos == d_b_back[i])
        {
            d_hero->addToBackpack(item, d_back+i);
            return true;
        }

        if (pos == d_b_ground[i])
        {
            GameMap::getInstance()->getTile(d_pos.x, d_pos.y)->addItem(item, d_ground+i);
            return true;
        }
    }
        
    return false;
}

bool ItemDialog::displayItem(Item* item)
{
    // Here we do not have any item on the button and we return 
    if (item==0) return false;

    // temporarily disable graphical updates
    PG_Application::SetBulkMode(true);
    
    d_l_item->SetText(item->getName().c_str());

    for (int i = 0; i < 3; i++)
        d_l_bonus[i]->SetText("");
    
    int pos = 0;
    char buffer[80];
    buffer[79] = '\0';

    // Now go through all possible enchantments and print a suitable description.
    // This is rather dumb, but I don't know a better solution.

    if (item->getBonus(Item::STRENGTH))
    {
        
        if (item->getValue(Item::STRENGTH) > 0)
        {
            snprintf(buffer, 79, _("+%i strength"), item->getValue(Item::STRENGTH));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
        }
        if (item->getValue(Item::STRENGTH) < 0)
        {
            snprintf(buffer, 79, _("%i strength"), item->getValue(Item::STRENGTH));
            d_l_bonus[pos]->SetFontColor(250, 50, 50);
        }
        
        d_l_bonus[pos]->SetText(buffer);
        pos++;
    }

    if (item->getBonus(Item::DEFENSE))
    {
        if (item->getValue(Item::DEFENSE) > 0)
        {
            snprintf(buffer, 79, _("+%i defense"), item->getValue(Item::DEFENSE));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
        }
        if (item->getValue(Item::DEFENSE) < 0)
        {
            snprintf(buffer, 79, _("%i defense"), item->getValue(Item::DEFENSE));
            d_l_bonus[pos]->SetFontColor(250, 50, 50);
        }
        
        d_l_bonus[pos]->SetText(buffer);
        pos++;
    }

    if (item->getBonus(Item::VITALITY))
    {
        if (item->getValue(Item::VITALITY) > 0)
        {
            snprintf(buffer, 79, _("+%i vitality"), item->getValue(Item::VITALITY));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
        }
        if (item->getValue(Item::VITALITY) < 0)
        {
            snprintf(buffer, 79, _("%i vitality"), item->getValue(Item::VITALITY));
            d_l_bonus[pos]->SetFontColor(250, 50, 50);
        }
        
        d_l_bonus[pos]->SetText(buffer);
        pos++;
        if (pos > 2)
            return true;
    }

    if (item->getBonus(Item::HP))
    {
        if (item->getValue(Item::HP) > 0)
        {
            snprintf(buffer, 79, _("+%i hitpoints"), item->getValue(Item::HP));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
        }
        if (item->getValue(Item::HP) < 0)
        {
            snprintf(buffer, 79, _("%i hitpoints"), item->getValue(Item::HP));
            d_l_bonus[pos]->SetFontColor(250, 50, 50);
        }
        
        d_l_bonus[pos]->SetText(buffer);
        pos++;
        if (pos > 2)
            return true;
    }

    if (item->getBonus(Item::MOVES))
    {
        if (item->getValue(Item::MOVES) > 0)
        {
            snprintf(buffer, 79, _("+%i vitality"), item->getValue(Item::MOVES));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
        }
        if (item->getValue(Item::MOVES) < 0)
        {
            snprintf(buffer, 79, _("%i vitality"), item->getValue(Item::MOVES));
            d_l_bonus[pos]->SetFontColor(250, 50, 50);
        }
        
        d_l_bonus[pos]->SetText(buffer);
        pos++;
        if (pos > 2)
            return true;
    }

    if (item->getBonus(Item::MOVE_BONUS))
    {
        // Can this cause trouble int and unsigned int and then testing bits?
        unsigned int bonus = static_cast<unsigned int>(item->getValue(Item::MOVE_BONUS));

        if (bonus & MAPTILE_WATER)
        {
            d_l_bonus[pos]->SetText(_("move on water"));
            d_l_bonus[pos]->SetFontColor(0, 0, 255);
            pos++;
            if (pos > 2)
                return true;
        }
        if (bonus & MAPTILE_FOREST)
        {
            d_l_bonus[pos]->SetText(_("move through forest"));
            d_l_bonus[pos]->SetFontColor(0, 160, 70);
            pos++;
            if (pos > 2)
                return true;
        }
        if (bonus & MAPTILE_HILL)
        {
            d_l_bonus[pos]->SetText(_("move over hills"));
            d_l_bonus[pos]->SetFontColor(150, 100, 80);
            pos++;
            if (pos > 2)
                return true;
        }
        if (bonus & MAPTILE_SWAMP)
        {
            d_l_bonus[pos]->SetText(_("move through swamp"));
            d_l_bonus[pos]->SetFontColor(150, 80, 50);
            pos++;
            if (pos > 2)
                return true;
        }
    }

    if (item->getBonus(Item::ARMY_BONUS))
    {
        unsigned int bonus = static_cast<unsigned int>(item->getValue(Item::ARMY_BONUS));

        if (bonus & ARMY_ANTICAVALRY)
        {
            d_l_bonus[pos]->SetText(_("double strength against cavalry"));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
            pos++;
            if (pos > 2)
                return true;
        }
        if (bonus & ARMY_CRITICAL)
        {
            d_l_bonus[pos]->SetText(_("can deal instant death"));
            d_l_bonus[pos]->SetFontColor(50, 250, 50);
            pos++;
            if (pos > 2)
                return true;
        }
    }
    
    PG_Application::SetBulkMode(false);
    Update();
    return true;
}

bool ItemDialog::eventKeyDown(const SDL_KeyboardEvent* key)
{
    switch(key->keysym.sym)
    {
        case SDLK_RETURN:
            okClicked(0,0,0,0);
            break;
        default:
            break;
    }
    return true;
}
