//  This program is free software; you can redistribute it d_terrainand/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <math.h>  
#include "MapGenerator.h"
#include "File.h"
#include "defs.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
#define debug(x)
#define maxlink 300
#define offmap(x,y) (y<0)||(y>=d_size)||(x<0)||(x>=d_size)

class portneeded
{
    public:
    int landid;
    int seaid;
    bool hasPort;
    int x,y; //possible site of port
    portneeded();
    ~portneeded();
};



portneeded::portneeded()
{
    landid=0;
    seaid=0;
    x=-1;y=0;
    hasPort=false;
}

portneeded::~portneeded()
{}

class routes
{
    public:
    int landid;
    int dest;
    int via ;
    routes();
    ~routes();
};




routes::routes()
{
    landid=0;
    dest=0;
    via=0;
}

routes::~routes()
{}



using namespace std;


class t_links {
  protected:
    long links[maxlink];  
    long closeup[maxlink];  
  public:
    void zero();
    void normalize ();
    int closeUp(int id);
    bool resetpeak(int base,int oldlink, int newlink);
    int getval(int p){return links[p];};
    int getclval(int p){return closeup[p];};
    void setval(int p, int val){ links[p]=val;};  
};

class t_sealinks : public t_links 
    {

  public:
    void normalize ();
    int closeUp(int id);
    bool resetpeak(int base,int oldlink, int newlink);
    long getval(int p){return links[abs(p)];}; 
    int getclval(int p){return closeup[abs(p)];};
    void setval(int p, long val){ links[abs(p)]=val;};  
};

void t_links::normalize()
{
    int loop, activ;
    bool found;
    for (loop=0;loop<maxlink; loop++)
    {
        if (links[loop]!=0)
        {
            activ=loop; 
            found=false;
            do
            {
                if (links[activ]==activ)  
                    found=true;
                else
                     activ=links[activ];
             }
             while (!found);
             links[loop]=links[activ];
         }    
    } 
}

int t_links::closeUp(int id)
{
    int nextindex=1;
    int loop;
    for ( loop=id;loop>1; loop--)
    {
        
        if  (links[loop]==loop) 
        {
            closeup[loop]=nextindex;
            nextindex++;
        }
    }    
    for ( loop=id;loop>1; loop--)
    {
        
        if  (links[loop]!=loop) 
            closeup[loop]=closeup[(links[loop])]; 
              
    }
    id=loop;
    return nextindex; 
}




bool t_links::resetpeak(int base,int oldlink, int newlink)
{
    bool found, inrange;
    inrange=true;
    found=false;
    do
    { 
        if (links[oldlink]==oldlink)  
            found=true;
        else
            oldlink=links[oldlink];
        if (oldlink>maxlink) inrange=false;
        
    }    
    while (!found);

    found=false;
    do
    {
        if (links[newlink]==newlink)  
            found=true;
        else
            newlink=links[newlink];
        if (newlink>maxlink) inrange=false;
    }
    while (!found);
    if (inrange)    
    { 
        if (newlink>=oldlink)
        {
            links[oldlink]=newlink;
            links[base]=newlink;
        }
        else
        {
            links[newlink]=oldlink;
            links[base]=oldlink;
        }
    }
    else
        cout<< "range error"<< newlink <<oldlink;
    return inrange;
}


void t_sealinks::normalize()
{
    int loop, activ;
    bool found;
    for (loop=0;loop<maxlink; loop++)
    {
        if (links[loop]!=0)
        {
            activ=loop; 
            found=false;
            do
            {
                if (abs(links[activ])==activ)  
                    found=true;
                else
                    activ=abs(links[activ]);
             }
             while (!found);
             links[loop]=links[activ];
         }
    } 
}

int t_sealinks::closeUp(int id)
{
    int nextid=-1;
    int loop;
    for ( loop=abs(id);loop>1; loop--)
    {
        
        if  (abs(links[loop])==loop) 
        {
            closeup[loop]=nextid;
            nextid--;
        }
    }    
    for ( loop=abs(id);loop>1; loop--)
    {
        
        if  (abs(links[loop])!=loop) 
            closeup[loop]=closeup[abs(links[loop])];     
    }
    id=loop;
    return nextid; 
}

bool t_sealinks::resetpeak(int base,int oldlink, int newlink)
{
    bool found,inrange;
    inrange=true;
    found=false;
    do
    {       
        if (links[abs(oldlink)]==oldlink)  
            found=true;
        else
            oldlink=links[abs(oldlink)];
        if (oldlink >maxlink) inrange=false;
    }
    while (!found);

    found=false;
    do
    {
        if (links[abs(newlink)]==newlink)  
            found=true;
        else
            newlink=links[abs(newlink)];
        if (newlink >maxlink) inrange=false;
    }
    while (!found);
    if (inrange)
    {
        if (newlink<=oldlink)
        {
            links[abs(oldlink)]=newlink;
            links[abs(base)]=newlink;
        }
        else
        {
            links[abs(newlink)]=oldlink;
            links[abs(base)]=oldlink;
        }
    }
    else
        cout<< "range error"<< newlink <<oldlink; 
    return inrange;
}


void t_links::zero()
    {
     int loop;
    for (loop=0;loop<maxlink; loop++)
    {
        links[loop]=0;
        closeup[loop]=0;
    }    
    }

enum e_Terrain {GRASS=0,WATER,FOREST,HILLS,MOUNTAINS,SWAMP,NOTTER};
//The above are the values used in the save file

e_Terrain operator ++ (e_Terrain t)
{
    switch (t) 
    {
	case GRASS: return WATER;
	case WATER: return FOREST;
	case FOREST: return HILLS;
	case HILLS:   return MOUNTAINS;
	case MOUNTAINS:   return SWAMP;
	case SWAMP:   return NOTTER;
	case NOTTER:   return NOTTER;
    }
return NOTTER;
}

e_Terrain d_terrainA [SWAMP];

e_Terrain char2terain(char c)
{
    if (c==CHARACTER_GRASS) return GRASS;
        else if (c==CHARACTER_WATER) return WATER;
            else if (c==CHARACTER_FOREST) return FOREST;
                else if  (c==CHARACTER_HILLS) return HILLS;
                    else if  (c==CHARACTER_MOUNTAINS) return MOUNTAINS;
                        else if  (c==CHARACTER_SWAMP) return SWAMP;  
                            else return NOTTER;
}


e_Terrain sucTer(e_Terrain t)
{
    switch (t) 
    {
	case GRASS: return WATER;
	case WATER: return FOREST;
	case FOREST: return HILLS;
	case HILLS:   return MOUNTAINS;
	case MOUNTAINS:   return SWAMP;
	case SWAMP:   return NOTTER;
	case NOTTER:   return NOTTER;
    }
return NOTTER;
}

MapGenerator::MapGenerator()
    //set reasonable default values
    :d_terrain(0), d_building(0), d_pswamp(2), d_pwater(25), d_pforest(3),
    d_phills(5), d_pmountains(5), d_conwat(2), d_nocities(11), d_notemples(9), d_noruins(20)

{
    // d_xdir[-1, 0, 1, 1, 1, 0,-1,-1];
   //  d_ydir[ 1, 1, 1, 0,-1,-1,-1, 0];
    d_xdir[0]=-1;d_xdir[1]=0;d_xdir[2]=1;d_xdir[3]=1;d_xdir[4]=1;d_xdir[5]=0;d_xdir[6]=-1;d_xdir[7]=-1;
    d_ydir[0]=1;d_ydir[1]=1;d_ydir[2]=1;d_ydir[3]=0;d_ydir[4]=-1;d_ydir[5]=-1;d_ydir[6]=-1;d_ydir[7]=0;
}

MapGenerator::~MapGenerator()
{
    if (d_terrain)
        delete[] d_terrain;
    if (d_building)
        delete[] d_building;
    if (d_l_mass)
        delete[] d_l_mass;    
}

int MapGenerator::setNoCities(int nocities)
{
    if (nocities <= 0)
        return -1;

    int tmp = d_nocities;
    d_nocities = nocities;
    return tmp;
}

int MapGenerator::setNoRuins(int noruins)
{
    if (noruins < 0)
        return -1;

    int tmp = d_noruins;
    d_noruins = noruins;
    return tmp;
}

int MapGenerator::setNoTemples(int notemples)
{
    if (notemples < 0)
        return -1;

    int tmp = d_notemples;
    d_notemples = notemples;
    return tmp;
}

void MapGenerator::setPercentages(int pwater, int pforest, int pswamp,
                                    int phills, int pmountains)
{
    if ((pswamp < 0) || (pwater < 0) || (pforest < 0) || (phills < 0)
        || (pmountains < 0))
        return;

    if (pswamp + pwater + pforest + phills + pmountains > 100)
        return;

    d_pswamp = pswamp;
    d_pwater = pwater;
    d_pforest = pforest;
    d_phills = phills;
    d_pmountains = pmountains;
}

/** 
 * Generates a random map . The map is stored as a char array of size 
 * 100x100. Each character stands for something. We use :
 * M = mountains
 * h = hills
 * ~ = water
 * $ = forest
 * . = plains
 * _ = swamps
 * C = city/castle
 * r = ruins
 * T = temple
 *   = nothing
 * c = part of city/castle
 */
void MapGenerator::MakeMap(int size)
{
    d_size = size;

    //initialize terrain and building arrays
    d_terrain = new char[size*size];
    d_building = new char[size*size];
    d_l_mass = new int [size*size];
    for (int i = 0; i < size; i++)
        for (int j = 0; j < size; j++)
            d_building[i*size + j] = ' ';

    cout <<_("Making random map") <<endl;
   
    cout <<_("Flatening Plains  ... 10%") <<endl;
    makePlains();
    cout <<_("Raining Water     ... 30%") <<endl;
    makeTerrain(CHARACTER_WATER,d_pwater, d_conwat);  
    cout <<_("Raising Hills     ... 20%") <<endl;
    makeTerrain(CHARACTER_HILLS,d_phills,1);
    cout <<_("Planting Forest   ... 40%") <<endl;
    makeTerrain(CHARACTER_FOREST,d_pforest,1);
    cout <<_("Watering Swamps   ... 50%") <<endl;
    makeTerrain(CHARACTER_SWAMP,d_pswamp,1);
    cout <<_("Normalizing       ... 60%") <<endl;
    normalize();
           
    int  nmrofSeas=0,nmrLands=0;
    continents(nmrLands, nmrofSeas);
    nmrofSeas=abs(nmrofSeas);
    findRoutes( nmrLands,nmrofSeas);
    cout <<_("Building Cities   ... 70%") <<endl;
    makeCities(d_nocities);
    cout <<_("Ruining Ruins     ... 80%") <<endl;
    makeBuildings(CHARACTER_RUIN,d_noruins);
    cout <<_("Spawning temples  ... 90%") <<endl;
    makeBuildings(CHARACTER_TEMPLE,d_notemples);
    cout <<_("Done making map   ... 100%") <<endl;
}

char* MapGenerator::getMap(int& size)
{
    size = d_size;

    return d_terrain;
}

char* MapGenerator::getBuildings(int& size)
{
    size = d_size;

    return d_building;
}

/* Nothing much to do here, just fill up everything with '.' 
 * as we are making the default canvas for the map.
 */
void MapGenerator::makePlains()
{
    for(int j = 0; j < d_size; j++)
        for(int i = 0; i < d_size; i++)
            d_terrain[j*d_size + i]=CHARACTER_GRASS;
}



/**
 * Makes Terrains.
 * The algo is as follows :
 * 1. Find a random starting location
 * 2. chose a random direction , if x is the starting location, the direction
 *    can be from 0-7 as follows :
 *    +-+-+-+
 *    |0|1|2|
 *    +-+-+-+
 *    |7|x|3|
 *    +-+-+-+
 *    |6|5|4|
 *    +-+-+-+
 *  or this CHECK!!  Fix Me
  *    +-+-+-+
 *    |6|5|4|
 *    +-+-+-+
 *    |7|x|3|
 *    +-+-+-+
 *    |0|1|2|
 *    +-+-+-+
 * 3. Check if there is some other terrain there ('.' = plains is okay)
 * 4. Reserve the location, and chose another direction
 */
void MapGenerator::makeTerrain(char c, int percent, int contin)
{
    // calculate the total number of tiles for this terrain
    int terrain = d_size*d_size*percent / 100;

    int placed = 0;  // total of current terrain placed so far 
    bool newStart;
    int done = 0;
    int loopcount = 0;
    int tmpx = 0;
    int tmpy = 0;
    int direction = 0;    
    int x,y;    // locations
    while(placed != terrain)
    {
        // find a random starting location that's grass
        do{
            x = rand()%(d_size);
            y = rand()%(d_size);
        }while( d_terrain[y*d_size + x]!=CHARACTER_GRASS);
        newStart=false;
        d_terrain[y*d_size + x]=c;// set starting position
        placed++;
        while((placed < terrain)&& !newStart)//full quatity of terrain not placed
        {
            //find direction
            done = 0,loopcount = 0;
            direction  = rand()%8; 
            while( !done )
            {
                // If blocked it tests for each possible direction in
                // turn moving clockwise.
                if(direction>7)  
                    direction=0;
                tmpx = x+d_xdir[direction];
                tmpy = y+d_ydir[direction];
                if(tmpx < d_size && tmpx >= 0 && tmpy < d_size && tmpy >= 0
                    && d_terrain[tmpy*d_size + tmpx]==CHARACTER_GRASS)
                    done =1;
                direction++;
                if(++loopcount > 7)
                    done =1;
            }
            if(loopcount>=7 || tmpx <0 || tmpy <0 || tmpx > (d_size) || tmpy >(d_size))  
            { 
                if (contin== 1)
                     break; // find a new random starting point
                else 
                {    

                    newStart= !seekPlain( c, tmpx, tmpy); //try to find starting point on the edge of the current block and if not exit loop
                }    
            }
            x=tmpx;
            y=tmpy;  
            d_terrain[y*d_size + x]=c;
            placed++;
        } // end while  exit with break if blocked

    }// end while - all of terain placed
        
}

/*  
**  seekPlain is called if the terrain generator hits a dead end.
**  This will often be because the active tile is surrounded by new laid
**  tiles of the current terrain "c".  seekPlain moves the active tile 
**  thru the current block of terrain c so that the current block is 
**  expanded.
**  the inner loop looks at each of the neighboring tiles in turn
**  the outer loop moves the active tile until either grass is
**  found or it is blocked.
*/
int MapGenerator::seekPlain(char c, int& x, int& y)
{
    int sdirection = 0;
    int sinitdir=0; 
    int tries = 0;
    int next,found,blocked;
    int tempx = 0;
    int tempy = 0;
    found = false; blocked=false;
    while(!found && !blocked)
    {
        //find direction
        next=false ;  
        tries = 0;
        sdirection  = rand()%8;
        sinitdir=sdirection;
        while( !found && !blocked && !next)
        { //looking at the adjacent tiles
            sdirection=sinitdir+tries;
            // If blocked it tests for each possible direction in
            // turn, moving clockwise.
            if(sdirection>7)
                sdirection -= 8;
                
            tempx = x+d_xdir[sdirection];
            tempy = y+d_ydir[sdirection];            
            if((tempx < d_size) && (tempx >=0) && (tempy<d_size) && (tempy >= 0)
                    && (d_terrain[tempy*d_size + tempx]==CHARACTER_GRASS))
            {    
                found =true;
                x=tempx; 
                y=tempy; 
            }    
            else  
            if((tempx<d_size) && (tempx>=0) && (tempy<d_size) && (tempy>=0)
                  && (d_terrain[tempy*d_size + tempx]==c) ) 
            {  //we are still on terain 'c' so look further for grass
                next=true;
                x=tempx; 
                y=tempy; 
            }  
            if(++tries > 7)
                    blocked =true;                

        }//end of loop that looks at ajacent tiles
        if(tries>=7 || tempx <0 || tempy <0 || tempx >= d_size || tempy >=d_size)
            blocked=true; // give up
    }
    return !blocked;
}




void MapGenerator::putcity(int x,int y, int&  city_count)
{
        d_building[y*d_size + x]    =CHARACTER_CITY_MAJOR;
        d_building[y*d_size + x+1]  =CHARACTER_CITY_MINOR;
        d_building[(y+1)*d_size + x]  =CHARACTER_CITY_MINOR;
        d_building[(y+1)*d_size + x+1]=CHARACTER_CITY_MINOR;

        //cities shall only sit on grass tiles
        d_terrain[y*d_size + x]       = CHARACTER_GRASS;
        d_terrain[y*d_size + x+1]     = CHARACTER_GRASS;
        d_terrain[(y+1)*d_size + x]   = CHARACTER_GRASS;
        d_terrain[(y+1)*d_size + x+1] = CHARACTER_GRASS;

        city_count ++;
}


void MapGenerator::makeCities(int cities)
{

    int x,y,city_count;
    int iterations = 0;
    
    // place the cities
    city_count = 0;
    while(city_count < cities)
    {
        x = rand()%(d_size-2);
        y = rand()%(d_size-2);
        placePorts( x, y, city_count);
        if((d_terrain[y*d_size +x] == CHARACTER_WATER
            || d_terrain[y*d_size + x+1] == CHARACTER_WATER 
            || d_terrain[(y+1)*d_size + x] == CHARACTER_WATER
            || d_terrain[(y+1)*d_size + x+1] == CHARACTER_WATER)
            && (iterations < 10))
        {
            iterations++;
            continue;
        }
        
        // check if we can put the building
        if(!canPutBuilding(x,y) &&
            ((iterations < 10)
            || (d_terrain[y*d_size+x] != CHARACTER_WATER)))
        {
            iterations++;
            continue;
        }
        
        putcity(x, y, city_count);
        
        iterations=0;

    }
    
}

void MapGenerator::makeBuildings(char c, int building)
{
    int x,y;
    int i = 0;
    int iterations = 0;
    
    while (i < building)
    {        
        x = rand()%d_size;
        y = rand()%d_size;
        
        //If number of iterations is smaller 10, look for a suitable
        //place. If this number is exceeded, place the temple on an
        //island if neccessary. Try to write down the condition. It is
        //difficult because I have contracted it a bit.
        if (!canPutBuilding(x, y) && 
            ((iterations < 10)
            || (d_terrain[y*d_size+x] != CHARACTER_WATER)))
        {
            iterations++;
            continue;
        }

        if (iterations >= 10)
            d_terrain[y*d_size + x] = CHARACTER_GRASS;
        d_building[y*d_size + x] = c;
        
        iterations = 0;
        i++;
    }
}

bool MapGenerator::tryToPlaceCity(int px,int py , int& city_count,bool& hasPort)
{
    if (canPutCity( px, py))
    { 
    //place a city
        putcity(px, py, city_count);
        hasPort=true;
        return true;
    } 
    else if (canPutCity( px-1, py))
    { 
    //place a city
        putcity(px-1, py, city_count);
        hasPort=true;
        return true;
    } 
    else if (canPutCity( px-1, py))
    { 
    //place a city
        putcity(px-1, py-1, city_count);
        hasPort=true;
        return true;
    } 
    else if (canPutCity( px-1, py))
    { 
    //place a city
        putcity(px, py-1, city_count);
        hasPort=true;
        return true;
    } 
  //  else cout << "failed c "<< px<<','<<py<<endl;
    return false;                   
                        
}

void MapGenerator::placePorts(int x,int y, int& city_count)
{
    int px,py;
    int lastwat=0;
    bool placed=false;
    if (d_l_mass[y*d_size + x]>0) //if it's not a sea tile
    {//1
        for (unsigned int loop = 0;loop< d_portneed.size(); loop++)
        {//2
            for (unsigned int loop2 = 0;loop2< d_portneed[loop].size(); loop2++)
            {//3
                if (d_portneed[loop][loop2].landid==d_l_mass[y*d_size + x])
                {//4
                     
                    px=d_portneed[loop][loop2].x;
                    py=d_portneed[loop][loop2].y;
                    if (!d_portneed[loop][loop2].hasPort)
                    {//5
                        lastwat=0;
                        walkCoast(px,py,d_portneed[loop][loop2].seaid,rand()%43, lastwat);
                        
                        placed=tryToPlaceCity( px, py,city_count,d_portneed[loop][loop2].hasPort);
 
                        if (!placed)
                        {
                            px=d_portneed[loop][loop2].x;
                            py=d_portneed[loop][loop2].y;
                            lastwat=7;
                            walkCoastAntiClock(px,py,d_portneed[loop][loop2].seaid,rand()%23,lastwat);
                            placed=tryToPlaceCity( px, py,city_count,d_portneed[loop][loop2].hasPort);
                        }
                        if (!placed)
                        {
                            px=d_portneed[loop][loop2].x;
                            py=d_portneed[loop][loop2].y;
                            bool walkable=true;
                            int walksteps=0;
                            lastwat=0;
                            placed=tryToPlaceCity( px, py,city_count,d_portneed[loop][loop2].hasPort);
                            while ((!placed)&& (walkable))
                            {                                                            
                                walkable=walkCoast(px,py, d_portneed[loop][loop2].seaid,1,lastwat);//now check every tile for a site
                                placed=tryToPlaceCity( px, py,city_count,d_portneed[loop][loop2].hasPort);
                                walksteps++;
                                if ((walksteps>200)||((px==d_portneed[loop][loop2].x)&&( py==d_portneed[loop][loop2].y)))
                                {
                                    walkable=false;
                                    cout<<" round the lake/island "<<walksteps<< endl;
                                }
                            }  
                            if (!placed)  cout << "gave up "<<px<<' '<<py<<endl; 
                        }    
                    }//5
                }  //4  
            }//3
        } //2   
    }//1
}


/** 
 * canPutBuilding
 * Checks if we can put a building at the specified place. 
 * If we are on a square with water, we cannot put it
 * nor if it is too close.
 * Checks for neighboring buildings yet sometimes these
 * are bang next to each other - why?
 * int x = x position
 * int y = y position
 */

bool MapGenerator::canPutBuilding(int x,int y)
{
  //Was unused
  //int count=0;        // stores the count of water squares.
 
    // if the building is on water, return false
    if (d_terrain[y*d_size +x] == CHARACTER_WATER)
        return false;
        
    //if the building is close to the map boundaries, return false
    if ((x < 1) || (x > d_size-4) || (y < 1) || (y > d_size-4))
        return false;
        
    //if there is another building too close, return false
    for (int i = -3; i <= 3; i++)
        for (int j = -3; j <= 3; j++)
        {
            if ((y+j < 0) || (y+j >= d_size) || (x+i < 0) || (x+i >= d_size))
                continue;
            if (d_building[(y+j)*d_size + x+i] != ' ')
                return false;
        }  
    // everything okay here ! return true
    return true;
    
}


/** 
 * walkCoast
 * moves along a coast line. Goes in a clock wise direction
 * around a lake anticlockwise round a island   
 * may miss the odd promentary.
 * lastwat is the direction from the tile advanced onto to
 * the water tile selected before that tile
 */


bool MapGenerator::walkCoast(int& x,int& y, int sea_id, int dist, int& lastwat)
{
    
    int steps = 0;
    int loopbreak;
    int nx,ny, dir;  //nx is the next x etc
    while (steps<dist)
    {//1
        dir=lastwat;
        nx = x+d_xdir[dir];
        ny = y+d_ydir[dir];
        if (offmap(nx,ny)) return false;
        loopbreak=0;
        while (d_l_mass[ny*d_size +nx]!=sea_id)
        {//2
            dir++;
            if(dir>7)
                dir = 0;
            nx = x+d_xdir[dir];
            ny = y+d_ydir[dir];
            loopbreak++;
            if (offmap(nx,ny)) return false;
            if  (loopbreak>10) {cout << "infinite sea  "<< steps<< endl; return false;}
        }//2
        loopbreak=0;
        while (d_l_mass[ny*d_size +nx]<1)
        {//2
            dir++;
            if(dir>7)
                dir = 0;
            nx = x+d_xdir[dir];
            ny = y+d_ydir[dir];
            if (offmap(nx,ny)) return false;
            loopbreak++;
            if  (loopbreak>10) {cout << "infinite land"<< endl; return false;}
        }//2
        if ((x==nx)&&(y==ny)) cout<< "walk on spot fdir"<< dir;
        lastwat=dir+dir%2-3;
        if (lastwat<0) lastwat+=8;
        x=nx;
        y=ny;
        steps++;
                    
    }//1
    return true;
}



/** 
 * walkCoastAntiClock
 * moves along a coast line in the opposite direction   
 * to walkCoast. may miss the odd promentary.
 */

bool MapGenerator::walkCoastAntiClock(int& x,int& y, int sea_id, int dist, int& lastwat)
{
    int steps = 0;
    int loopbreak;
    
    int nx,ny, dir;  //nx is the next x etc
    while (steps<=dist)
    {//1
        dir=lastwat;
        nx = x+d_xdir[dir];
        ny = y+d_ydir[dir];
        if (offmap(nx,ny)) return false;
        loopbreak=0;
        while (d_l_mass[ny*d_size +nx]!=sea_id)
        {//2
            dir--;
            if(dir<0)
                dir = 7;
            nx = x+d_xdir[dir];
            ny = y+d_ydir[dir];
            loopbreak++;
            if (offmap(nx,ny)) return false;
            if  (loopbreak>10) {cout << "infinite sea  "<< steps<< endl; return false;}
        }//2
        loopbreak=0;
        while (d_l_mass[ny*d_size +nx]<1)
        {//2
            dir--;
            if(dir<0)
                dir = 7;
            nx = x+d_xdir[dir];
            ny = y+d_ydir[dir];
            if (offmap(nx,ny)) return false;
            loopbreak--;
            if  (loopbreak<-10) {cout << "infinite land ac"<< endl; return false;}
        }//2
        lastwat=dir-dir%2-5;
        if (lastwat<0) lastwat+=8;
        x=nx;
        y=ny;
        steps++;                   
    }//1
    return true;
}


/** 
 * canPutCity
 * Checks if the cordinates mark a tile that can be the  
 * major tile (the northwest tile) of a city.
 * int x = x position
 * int y = y position
 */

bool MapGenerator::canPutCity(int x,int y)
{
    if ((canPutBuilding(x,y))&&(canPutBuilding(x+1,y))&&(canPutBuilding(x,y+1))&&(canPutBuilding(x+1,y+1)))
    return true;
    else
    return false;
    
}



void MapGenerator::normalize()
{   
    
    int ajacentTer [6];
    e_Terrain curTer, ajTer, tloop;
    curTer=SWAMP;
    
// Go thru every tile bar the outer edge
    for( int globy=1;globy<(d_size-2);globy++)
    {
        for(int globx=1;globx<(d_size-2);globx++)
        {
            // zero the count of ajacent tiles
            for (tloop=GRASS;tloop<=SWAMP;tloop=sucTer(tloop))
                ajacentTer[tloop]=0;
            // with the current tile check all neighbors
            for(int locx=globx-1;locx<=globx+1;locx++)
            {
                for(int locy=globy-1;locy<=globy+1;locy++)
                { 
                     ajTer=char2terain(d_terrain[locy*d_size +locx]);
                     if ((globy==locy )&&(globx==locx))
                         curTer= ajTer;
                         
                     else                        
                         ajacentTer[ajTer]++; 
                }
            }
            if (curTer==WATER) // For the moment only water is normalized
            {
                if (ajacentTer[curTer]==0) d_terrain[globy*d_size +globx]=CHARACTER_GRASS;
                else if (ajacentTer[curTer]==1) 
                    {if  (rand()%100 < 95 )d_terrain[globy*d_size +globx]=CHARACTER_GRASS;}
                else if (ajacentTer[curTer]==2) 
                    {if  (rand()%100 < 70 )d_terrain[globy*d_size +globx]=CHARACTER_GRASS;}
                else if (ajacentTer[curTer]==3) 
                    {if  (rand()%100 < 40 )d_terrain[globy*d_size +globx]=CHARACTER_GRASS;}
                
            }
            else 
            {
                if (ajacentTer[WATER]==8) d_terrain[globy*d_size +globx]=CHARACTER_WATER;
                else  
                {
                    if ((ajacentTer[WATER]==7)&&(rand()%100 < 70 )) d_terrain[globy*d_size +globx]=CHARACTER_WATER;
                     else if ((ajacentTer[WATER]==6)&&(rand()%100 < 40 )) d_terrain[globy*d_size +globx]=CHARACTER_WATER;
                 }
                
             }
        }
     }  
}


/*
 * We need to designate the seperate landmasses/islands and the 
 * seas that connect them so that we know where we need to place ports
 * to ensure that a player can build ships to get his heros from one 
 * landmass to another.
 * In the first run it designates all sea as -1 and all land as 1.
 * The second run it gives seperate ids for each seperate land and
 * sea area.  Inevitably it will often give a new id to a bit of
 * aparently isolated land/sea which turns out to merley a section
 * of an already identified land/sea.
 * Each time it is discovered that two patches of land (or sea)
 * are in fact the same this link is recorded.  This is done 
 * using t_links and t_sealinks.  The reason
 * we need a different class for sea from land is that sea is 
 * distiguished from land by making the id negative
 * We need to designate seperate seas as well as land
 * to ensure that a "port" is not simply a city with a pond next to it
*/
 

  
void MapGenerator::continents(int& nmrLands, int&  nmrSeas )
{   
    bool peak;
    long id, s_id; //the number that will designate the next found landmass - s_id for seas
    id=2;  s_id=-2 ;// for now 1 is for land not yet assigned to a continent so 2 is first continent
    t_links landLinks; 
    t_sealinks seaLinks;
    landLinks.zero();
    seaLinks.zero();
//  Now to map out continents - first what's land and what's not
        for(int globy=0;globy<(d_size);globy++)
    {//1
        
        for(int globx=0;globx<(d_size);globx++)
        {//2
            d_l_mass[globy*d_size +globx]=0;          
            if (CHARACTER_WATER==d_terrain[globy*d_size +globx])
                d_l_mass[globy*d_size +globx]=-1;
            else
                d_l_mass[globy*d_size +globx]=1;

         }//2
    }//1



     //set top corner
    if (d_l_mass[0]==1) 
    {    
        d_l_mass[0]=id;
        id++;
    }    
    else
    {   
        d_l_mass[0]=s_id;
        s_id--;
    }    
     

//down the side  
    for (int globy=1;globy<(d_size);globy++) 
    {//1
        if  (((0>d_l_mass[globy*d_size] )&&(0>d_l_mass[(globy-1)*d_size]))||((0<d_l_mass[globy*d_size] )&&(0<d_l_mass[(globy-1)*d_size])))//  both the same
        {//2
            d_l_mass[globy*d_size]=d_l_mass[(globy-1)*d_size];
         }//2
         else if (0>d_l_mass[globy*d_size] ) //water
         {//2
             seaLinks.setval(s_id,s_id); 
             d_l_mass[globy*d_size]=s_id; 
             s_id--;
         } //2
         else //must be isloated land
         { //2
             landLinks.setval(id,id); 
             d_l_mass[globy*d_size]=id; 
             id++;
          } //2
     }//1
//along the top
    for (int globx=1;globx<(d_size);globx++) 
    {//1
        if  (((0>d_l_mass[globx] )&&(0>d_l_mass[globx-1]))||((0<d_l_mass[globx] )&&(0<d_l_mass[globx-1])))//  both the same
        {//2
            d_l_mass[globx]=d_l_mass[globx-1];
         }//2
         else if (0>d_l_mass[globx] ) //water
         {//2
             seaLinks.setval(s_id,s_id); 
             d_l_mass[globx]=s_id; 
             s_id--;
         } //2
         else 
         { //2
             landLinks.setval(id,id); 
             d_l_mass[globx]=id; //must be isloated land
             id++;
          } //2
     }//1
//along the bottom
    for (int globx=1;globx<(d_size);globx++) 
    {//1
        if  (((0>d_l_mass[(d_size*(d_size-1))+globx] )&&(0>d_l_mass[(d_size*(d_size-1))+globx-1]))||((0<d_l_mass[(d_size*(d_size-1))+globx] )&&(0<d_l_mass[(d_size*(d_size-1))+globx-1])))//  botht he same
        {//2
            d_l_mass[(d_size*(d_size-1))+globx]=d_l_mass[(d_size*(d_size-1))+globx-1];
         }//2
         else if (0>d_l_mass[(d_size*(d_size-1))+globx] ) //water
         {//2
             seaLinks.setval(s_id,s_id); 
             d_l_mass[(d_size*(d_size-1))+globx]=s_id; 
             s_id--;
         } //2
         else 
         { //2
             landLinks.setval(id,id); 
             d_l_mass[(d_size*(d_size-1))+globx]=id; //must be isloated land
             id++;
          } //2
     }//1
// down the far side
    for (int globy=1;globy<(d_size);globy++) 
    {
        if  (((0>d_l_mass[globy*d_size+d_size-1] )&&(0>d_l_mass[(globy-1)*d_size+d_size-1]))||((0<d_l_mass[globy*d_size+d_size-1] )&&(0<d_l_mass[(globy-1)*d_size+d_size-1])))//  both the same
        {
            d_l_mass[globy*d_size+d_size-1]=d_l_mass[(globy-1)*d_size+d_size-1];
         }
         else if (0>d_l_mass[globy*d_size+d_size-1] ) //water
         {
             seaLinks.setval(s_id,s_id); 
             d_l_mass[globy*d_size+d_size-1]=s_id; 
             s_id--;
         } 
         else //must be isloated land
         { //2
             landLinks.setval(id,id); 
             d_l_mass[globy*d_size+d_size-1]=id; 
             id++;
          } //2
     }//1

//  Now for the bulk of the terrain
    for(int globy=1;globy<(d_size-1);globy++)
    {//1

        for(int globx=1;globx<(d_size-1);globx++)
        {//2 
            {//3
                for(int locx=globx-1;locx<=globx+1;locx++)
                    for(int locy=globy-1;locy<=globy+1;locy++)
                    {//4 
                        
                        if ((d_l_mass[locy*d_size +locx]>0)&&(d_l_mass[globy*d_size +globx]>0))  //both land
                        { //5
// compare ids of with the neighboring tiles and set to the highest
                            if (d_l_mass[globy*d_size +globx]>d_l_mass[locy*d_size +locx]) 
                            {//6 active tile is biggest
                                if (1==d_l_mass[locy*d_size +locx])
                                     landLinks.setval( d_l_mass[locy*d_size +locx],landLinks.getval(d_l_mass[globy*d_size +globx]));
                                 else 
                                 {//7
                                     peak=landLinks.resetpeak(d_l_mass[globy*d_size +globx],landLinks.getval(d_l_mass[globy*d_size +globx]),landLinks.getval(d_l_mass[locy*d_size +locx]));                                
                                     d_l_mass[locy*d_size +locx]= d_l_mass[globy*d_size +globx];
                                 }//7
                             } //6
                            else  
                            {//6  the other tile has a bigger value
                                 if  (1==d_l_mass[globy*d_size +globx])
                                     landLinks.setval(d_l_mass[globy*d_size +globx],landLinks.getval(d_l_mass[locy*d_size +locx])); 
                                 else
                                 { //7

                                     peak=landLinks.resetpeak(d_l_mass[globy*d_size +globx],landLinks.getval(d_l_mass[globy*d_size +globx]),landLinks.getval(d_l_mass[locy*d_size +locx]));
                                     if (!peak) cout << " out of range";
                                 }//7
                                 d_l_mass[globy*d_size +globx]=  d_l_mass[locy*d_size +locx]; 
                             }//6
                         }  //5
                         else if ((d_l_mass[locy*d_size +locx]<1)&&(d_l_mass[globy*d_size +globx]<1)) //both sea
                         {//5
                            if (d_l_mass[globy*d_size +globx]<d_l_mass[locy*d_size +locx]) 
                            {//6 active tile is most negative
                                 if  (-1==d_l_mass[locy*d_size +locx])
                                 {//7
                                     seaLinks.setval(d_l_mass[locy*d_size +locx],seaLinks.getval(d_l_mass[globy*d_size +globx])); 
                                     d_l_mass[locy*d_size +locx]= d_l_mass[globy*d_size +globx];
                                 }//7
                                 else 
                                 {//7
                                     peak=seaLinks.resetpeak(d_l_mass[globy*d_size +globx],seaLinks.getval(d_l_mass[globy*d_size +globx]),seaLinks.getval(d_l_mass[locy*d_size +locx])); 
                                     if (!peak) cout << " out of range";
                                     d_l_mass[locy*d_size +locx]= d_l_mass[globy*d_size +globx];
                                 }//7
                             } //6
                            else  //active tile is less negative
                            {//6
                                 if  (-1==d_l_mass[globy*d_size +globx])
                                     seaLinks.setval(d_l_mass[globy*d_size +globx],seaLinks.getval(d_l_mass[locy*d_size +locx])); 
                                 else 
                                 {//7

                                     peak=seaLinks.resetpeak(d_l_mass[globy*d_size +globx],seaLinks.getval(d_l_mass[globy*d_size +globx]),seaLinks.getval(d_l_mass[locy*d_size +locx])); 
                                     if (!peak) cout << " out of range";
                                 }   //7
                                 d_l_mass[globy*d_size +globx]=  d_l_mass[locy*d_size +locx]; 
                             }//6 
                         }//5
                     }  //4  
// if after that the tile still has an id of 1 then pos new continent      
                if (d_l_mass[globy*d_size +globx]==1) 
                {  //4
                    d_l_mass[globy*d_size +globx]=id;
                    landLinks.setval(id,id);
                    id++;
                 }  //4
                 else if (d_l_mass[globy*d_size +globx]==-1) // or new sea
                {  //4
                    d_l_mass[globy*d_size +globx]=s_id;
                    seaLinks.setval(s_id,s_id); 
                    s_id--;
                 }  //4
            }//3
         } //2   
                 
    }//1
   
    landLinks.normalize();
    seaLinks.normalize();
    id=landLinks.closeUp(id);
    s_id=seaLinks.closeUp(s_id);
//  2nd pass - reasign secondary blocks on the same continent to the one with the highest value 
     for(int globy=0;globy<(d_size);globy++)
    {//1
        for(int globx=0;globx<(d_size);globx++)
        {//2
             if (d_l_mass[globy*d_size +globx]>0)               
               d_l_mass[globy*d_size +globx]=landLinks.getclval(d_l_mass[globy*d_size +globx]);
             else
                 d_l_mass[globy*d_size +globx]=seaLinks.getclval(d_l_mass[globy*d_size +globx]);
         }//2
     }//1


    nmrSeas= abs(s_id)-1;
    nmrLands=id-1;

}

void MapGenerator::findRoutes(int nmrLands ,int nmrSeas)// Lands is short for LandMasses
{
    cout <<"findRoutes"<< " n l "<<nmrLands<< " ns "<< nmrSeas;
    unsigned int node;
    portneeded sln;  
    vecports n(0); 
    vector<vecports>   seas(nmrSeas,n);
    bool found;    

/* seas is a vector of integers.  The first index is related to the id of the sea,
*  that is the absolute value -1.  The integer value is the id of the land mass. 
*  This loop cycles thru the map looking for land tiles next to sea tiles
*  it then adds the land tile id to the list of land masses conected to the sea */
   
    for(int globy=1;globy<(d_size-1);globy++)
    {//1
        for(int globx=1;globx<(d_size-1);globx++)
        {//2
            for(int locx=globx-1;locx<=globx+1;locx++)
                for(int locy=globy-1;locy<=globy+1;locy++) 
                {//3

                    if ((d_l_mass[globy*d_size +globx]<0)&& (d_l_mass[locy*d_size +locx]>0) )// reference tile (global) is sea, neighbor is land
                    {//4
                        found=false;
                        node=1; 
                        unsigned int nsize  ;           
                        nsize= seas[abs(d_l_mass[globy*d_size +globx])-1].size();
                       
                        if (nsize>0)
                        {//5
                            while ((!found)&&(node<=nsize))//<=
                            {//6
                                if (seas[abs(d_l_mass[globy*d_size +globx])-1][node-1].landid==d_l_mass[locy*d_size +locx])
                                {//7
                                    found=true;
                                }//7
                                else 
                                    node++;
                            }//6
                        }//5
                        if (!found)
                        {//5       
                                sln.landid= d_l_mass[locy*d_size +locx];                       
                                sln.x=locx;
                                sln.y=locy;
                                seas[abs(d_l_mass[globy*d_size +globx])-1].push_back(sln);                             
                                
                        }  //5 
    
                        
                    }//4
                    else  if ((d_l_mass[globy*d_size +globx]>0)&& (d_l_mass[locy*d_size +locx]<0) )// reference tile (global) is land, neighbor is sea
                    {//4  ToDo: this section is only needed to check for small seas on the edges yet it checks the whole area because the
                    //rest has already been done.
                        found=false;
                        node=1;
 
                        unsigned int nsize  ;           
                        nsize= seas[abs(d_l_mass[locy*d_size +locx])-1].size();
                       
                        if (nsize>0)
                        {//5
                            while ((!found)&&(node<=nsize))
                            {//6
                                if (seas[abs(d_l_mass[locy*d_size +locx])-1][node-1].landid==d_l_mass[globy*d_size +globx])
                                {//7
                                    found=true;
                                }//7
                                else 
                                    node++;
                            }//6
                        }//5
                        if (!found)
                        {//5     
                                sln.x=globx;
                                sln.y=globy;   
                                sln.landid= d_l_mass[globy*d_size +globx]  ;                 
                                seas[abs(d_l_mass[locy*d_size +locx])-1].push_back(sln);

                        }  //5 
                    }//4
 
                }//3
        }  //2      
    }  //1    
    typedef  vector<routes> veclanes;
    routes nr;
   
    veclanes  lanes1d;
    vector<veclanes>   lanes2d;
    unsigned int nsize;
    if (seas.size()>0)
        for (unsigned int sloop=0;sloop<seas.size(); sloop++)//sloop is each sea
        {

            if (seas[sloop].size()>unsigned (1))  //ie not a puddle conected to only one land-mass
                for (unsigned int lloop=0;lloop<seas[sloop].size();lloop++)             
                {
                   
                    nr.landid=seas[sloop][lloop].landid;
                    nr.via=-1*(sloop+1);                   
                    found=false;
                    node=0;
                    nsize=lanes2d.size();
                    while ((!found)&&(node<nsize))
                    {
                        if (seas[sloop][lloop].landid==(lanes2d[node][0].landid))// is the active land id one that has already been assigned?
                             found=true;
                        else node++;
                    }
                    if (lloop !=0)
                    nr.dest=seas[sloop][0].landid;
                    else
                    nr.dest=seas[sloop][1].landid; 
                    if (!found)
                    {
                        lanes1d.erase(lanes1d.begin(),lanes1d.end());
                        lanes1d.push_back(nr);
                        lanes2d.push_back(lanes1d);
                        
                    }
                    else lanes2d[node].push_back(nr);
                    
                    
                    
                    int initdest=1;
                    if (0==lloop )
                        initdest=2;
                    for (unsigned int dest=initdest;dest<seas[sloop].size();dest++)
                        if (dest!=lloop)//ie not a connection with itself
                        {    
                            nr.dest=seas[sloop][dest].landid;
                            lanes2d[node].push_back(nr);                       
                        }

                    
                }
        } 
        
//the seas that each land mass has to connect to by a port
    portneeded np;
   
    vecports  ports1d;

    if (seas.size()>1)
        for (unsigned int sloop=0;sloop<seas.size(); sloop++)//sloop is each sea
        {      
            if (seas[sloop].size()>1)  //ie not a puddle conected to only one land-mass
                for (unsigned int lloop=0;lloop<seas[sloop].size();lloop++)             
                {                  
                    np.landid=seas[sloop][lloop].landid;
                    np.x=seas[sloop][lloop].x;np.y=seas[sloop][lloop].y;
                    np.seaid=-1*(sloop+1);                   
                    found=false;
                    node=0;

                    while ((!found)&&(node<d_portneed.size()))
                    {
                        if (seas[sloop][lloop].landid==(d_portneed[node][0].landid))// is the active land id one that has already been assigned?
                             found=true;
                        else node++;
                    }

                    if (!found)
                    {

                        ports1d.erase(ports1d.begin(),ports1d.end());
                        ports1d.push_back(np);
                        d_portneed.push_back(ports1d);

                    }
                    else d_portneed[node].push_back(np);
                }
        } 


}
