//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*

#include <pglabel.h>
#include <pgfilearchive.h>
#include "File.h"
#include "Configuration.h"
#include "OptionsDialog.h"
#include "defs.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

OptionsDialog::OptionsDialog(PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, _("Game settings"), WF_MODAL)
{
    debug("OptionsDialog()");

    d_cb_smooth_scrolling = new PG_CheckButton(this, 0, PG_Rect(10, 40, Width() - 20, 30), _("Smooth scrolling"));
    d_cb_show_next_player = new PG_CheckButton(this, 0, PG_Rect(10, 70, Width() - 20, 30), _("Show next player popup"));

    /* Set initial state */
    if(Configuration::s_smoothScrolling)
        d_cb_smooth_scrolling->SetPressed();

    if(Configuration::s_showNextPlayer)
        d_cb_show_next_player->SetPressed();

    d_b_ok = new PG_Button(this, BTN_ID_OK, PG_Rect(20, Height() - 40, 60, 30), _("OK"));
    d_b_cancel = new PG_Button(this, BTN_ID_CANCEL, PG_Rect(90, Height() - 40, 80, 30), _("Cancel"));
    d_b_help = new PG_Button(this, BTN_ID_HELP, PG_Rect(180, Height() - 40, 70, 30), _("Help"));

    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&OptionsDialog::okClicked);
    d_b_cancel->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&OptionsDialog::cancelClicked);
    d_b_help->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&OptionsDialog::helpClicked);
}

OptionsDialog::~OptionsDialog()
{
    delete d_b_ok;
    delete d_b_cancel;
    delete d_b_help;
    delete d_cb_show_next_player;
    delete d_cb_smooth_scrolling;
}

PARAGUI_CALLBACK(OptionsDialog::okClicked)
{
    /* Transfer button state to configuration */
    Configuration::s_showNextPlayer = d_cb_show_next_player->GetPressed();
    Configuration::s_smoothScrolling = d_cb_smooth_scrolling->GetPressed();
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}
PARAGUI_CALLBACK(OptionsDialog::cancelClicked)
{
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}
PARAGUI_CALLBACK(OptionsDialog::helpClicked)
{
    std::cerr <<_("Not yet implemented");
    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

// End of file
