//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "QKillHero.h"
#include "signals.h"
#include <iostream>
#include <sstream>
#include <assert.h>
#include "stacklist.h"
#include <algorithm>
//=======================================================================
/** \brief An auxiliary function, maybe some day it'll be moved
           to some more general place.
 */
static void getEnemyHeroes(std::vector<Hero*>& dst);


using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

//=======================================================================
QuestKillHero::QuestKillHero(QuestsManager& mgr, 
                             int hero) : Quest(mgr, hero)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestKillHero::dyingArmy));
    // now let's move to the next player and find his hero,
    // after finishing with the next player and having not
    // chosen a hero procede to the next player... and so on
    // until we make a decision

    // determine the 'other' heroes count:

    Hero *hunted = chooseToKill();

    // if there are no heroes belonging to other players, 
    // we exit the program
    assert(hunted);
    d_hero_to_kill = hunted->getId();
    _initDescription(d_description, hunted);
}
//=======================================================================
void QuestKillHero::_initDescription(std::string& d, Hero *hero)
{
    char buffer[101]; buffer[100]='\0';
    snprintf(buffer, 100, _("Kill the hero named %s, servant of player %s"),
            hero->getName().c_str(), hero->getOwner()->getName().c_str());
    
    d = std::string(buffer);
}
//=======================================================================
bool QuestKillHero::isFeasible()
{
    std::vector<Hero*> heroes;
    getEnemyHeroes(heroes);
    return (heroes.empty() == 0);
}
//=======================================================================
QuestKillHero::QuestKillHero(QuestsManager& q_mgr, int heroId,
                             XML_Helper* helper) : Quest(q_mgr, heroId)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestKillHero::dyingArmy));

    // read hunted hero id
    helper->getData(d_hero_to_kill, "to_kill");

    // find this hero object:
    debug("load: hero_to_kill = " << d_hero_to_kill);

    Hero *hero = getHeroById(d_hero_to_kill);

    assert(hero);
    // finish initialization:
    _initDescription(d_description, hero);
}
//=======================================================================
Hero* QuestKillHero::chooseToKill()
{
    std::vector<Hero*> heroes;
    getEnemyHeroes(heroes);
    if (heroes.empty())
        return NULL;
    // ok - pick up a hero:
    unsigned int size = heroes.size();
    unsigned int idx = (rand() % size);
    return heroes[idx];
}
//=======================================================================
void QuestKillHero::getProgress(std::string& dst)
{
    dst += _("You're still searching for him...");

    Stack* stack = 0;
    
    // add info about the location of the hunted:
    getHeroById(d_hero_to_kill, &stack);

    debug("Hunted hero stack is = " << stack);
    std::stringstream s;
    s << _("Seen lately near (") << stack->getPos().x << ", "
      << stack->getPos().y << ")";
    dst += s.str();
}
//=======================================================================
void QuestKillHero::dyingArmy(Army *army, culpritsType culprits)
{
    if (!pending)
        return;
        
    // is it a hero we were hunting for? */
    int id = army->getId();
    if (id == (int) d_hero_to_kill)
    {
        culpritsType::iterator it;
        // ok... now check if he was killed by our hero */
        unsigned int ourHero = getHeroId();

        if (culprits.size()==0) 
        {
            /*The Hero was killed by a stack without heroes so the quest expires*/
            debug("SORRY: YOUR QUEST 'KILL HERO' HS EXPIRED BECAUSE THE HERO TO KILL WAS KILLED BY ONOTHER ONE");
            questExpired();                 
            return;
        }

        for (it = culprits.begin(); it != culprits.end(); it++)
        {
            if ((*it) == ourHero)
            {
                debug("CONGRATULATIONS: QUEST 'KILL HERO' IS COMPLETED!");
                questCompleted();                 
                break;
            }
            else 
            {
                debug("SORRY: YOUR QUEST 'KILL HERO' HS EXPIRED BECAUSE THE HERO TO KILL WAS KILLED BY ONOTHER ONE");
                questExpired();                 
                break;
            }
        }
    }
}
//=======================================================================
void QuestKillHero::getSuccessMsg(std::queue<std::string>& msgs)
{
    std::string name = getHeroById(d_hero_to_kill, 0)->getName();
    char buffer[101]; buffer[100]='\0';
    snprintf(buffer, 100, _("You slayed the wicked hero %s. He deserved that!"), name.c_str());
    msgs.push(std::string(buffer));
}
//=======================================================================
void QuestKillHero::getExpiredMsg(std::queue<std::string>& msgs)
{
    char buffer[101]; buffer[100]='\0';
    snprintf(buffer, 100, _("You could not slay the wicked hero %s"),
            getHeroById(d_hero_to_kill, 0)->getName().c_str());
    msgs.push(std::string(buffer));
    msgs.push(_("He was slain by other fellows!"));
}
//=======================================================================
bool QuestKillHero::save(XML_Helper* helper)
{
    return helper->saveData("to_kill", d_hero_to_kill);
}
//=======================================================================
static void getEnemyHeroes(std::vector<Hero*>& dst)
{
    // loop through the player's unit
    // for every hero check any pending quests
    Playerlist* list = Playerlist::getInstance();
    Player *player  = Playerlist::getActiveplayer();

    // we start from the next player after us:
    Playerlist::iterator pit = find(list->begin(), list->end(), player);
    do
    {
        pit++;
        if (pit == list->end())
            pit=list->begin();
        if ((*pit) != player)
        {
            // get all heroes from all player's stacks
            Stacklist* stacklist = (*pit)->getStacklist();
            for (Stacklist::const_iterator it = stacklist->begin(); 
                it != stacklist->end(); it++)
            {
                for (Stack::iterator sit = (*it)->begin(); 
                    sit != (*it)->end(); sit++)
                {
                    if ((*sit)->isHero()) 
                        dst.push_back(dynamic_cast<Hero*>(*sit));
                }
            }
        }
    }
    while ((*pit) != player);
}
