//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef __QUEST_KILL_HERO_H
#define __QUEST_KILL_HERO_H

#include "Quest.h"
#include <sigc++/sigc++.h>
#include "hero.h"
#include "signals.h"
#include "playerlist.h"

class QuestKillHero : public Quest, public SigC::Object
{
    public:

        QuestKillHero(QuestsManager& q_mgr, int heroId);

        /** \brief Constructor - create a new quest from
                   saved data */
        QuestKillHero(QuestsManager& q_mgr, int heroId,
                      XML_Helper* helper);

        /**
         * \brief Return whether quest is possible at all
         */
        static bool isFeasible();

        /**
         * \brief Get progress information 
         *
         * \param s here we append the progress information
         */
         void getProgress(std::string& s);

        /**
         * \brief Provide the lines of the message describing
                  the quest completion.
         */
         void getSuccessMsg(std::queue<std::string>& msgs);

        /**
         * \brief Provide the lines of the message describing
                  the quest expiration.
         */
         void getExpiredMsg(std::queue<std::string>& msgs);

    private:

        /** slot that would receive the sdyingArmy (signal) */
        void dyingArmy(Army *stack, culpritsType culprits);

        bool save(XML_Helper* helper);

        /** \brief Choose a hero to be killed */
        Hero *chooseToKill();
        /** \brief Choose a hero to be killed out of the player's
                   heroes
         */
        Hero* chooseFromPlayer(Playerlist::iterator& pit);

        void _initDescription(std::string& d, Hero *hero);

        /** hero id (the target) to be searched by the hero */
        int d_hero_to_kill;
};

#endif
