#ifndef __QUEST_RUIN_SEARCH_H
#define __QUEST_RUIN_SEARCH_H

#include "Quest.h"
#include <sigc++/sigc++.h>
#include "xmlhelper.h"
#include "ruin.h"

class QuestRuinSearch : public Quest, public SigC::Object
{
    public:

        /** \brief Constructor - create a new quest */
        QuestRuinSearch(QuestsManager& q_mgr, int heroId);

        /** \brief Constructor - create a new quest from
                   saved data */
        QuestRuinSearch(QuestsManager& q_mgr, int heroId,
                        XML_Helper* helper);

        /**
         * \brief Get progress information 
         *
         * \param s here we append the progress information
         */
         void getProgress(std::string& s);

        /**
         * \brief Provide the lines of the message describing
                  the quest completion.
         */
         void getSuccessMsg(std::queue<std::string>& msgs);

         /**
         * \brief Provide the lines of the message describing
                  the quest completion.
         */
        void getExpiredMsg(std::queue<std::string>& msgs);


         /**
          * \brief Checks if such a quest is possible at all.
          */
         static bool isFeasible();


    private:

         /** \brief Count the unsearched ruins */
         static unsigned int _howManyUnsearched();

         bool save(XML_Helper* helper);

         /** \brief Make quest description from the ruin we'll search */
         void _initDescription(std::string& d, Ruin& ruin);
         /** slot that would receive the sruinSearched (signal) */
         void ruinSearched(int ruin_id, int hero_id);
         /** ruin id to be searched by the hero */
         int d_ruin_id;
};

#endif
