//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "QuestCompletedDialog.h"
#include "Quest.h"
#include "player.h"
#include "defs.h"
#include <stdlib.h>

static const unsigned int WIDTH       = 400;
static const unsigned int HEIGHT      = 340;
static const unsigned int MARGIN_X    = 10;
static const unsigned int LINE_WIDTH  = WIDTH - 2 * MARGIN_X;
static const unsigned int LINE_HEIGHT = 20;

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

QuestCompletedDialog::QuestCompletedDialog(PG_Widget* parent, Quest *quest)
	:PG_Window(parent,
            PG_Rect((PG_Application::GetScreenWidth()-WIDTH)/2,
            (PG_Application::GetScreenHeight()-HEIGHT)/2, WIDTH, HEIGHT),
            _("CONGRATULATIONS!"), WF_MODAL)
{

    queue<std::string> msgs;
    quest->getSuccessMsg(msgs);
    unsigned int y = 40;
    char buffer[101];
    buffer[100]='\0';
    std::string s;

    snprintf(buffer, 100, _("Your Hero %s completed the quest."),
            getHeroById(quest->getHeroId())->getName().c_str());

    PG_Label *msg = new PG_Label(this, PG_Rect(MARGIN_X, y, 
                       LINE_WIDTH, LINE_HEIGHT), 
                       buffer);
    msg->SetAlignment(PG_TA_CENTER);
    msg->SetFontColor(0, 255, 0);
    d_msgs.push(msg);

    y = 70;

    while (msgs.empty() == false)
    {
        s =  msgs.front();
	    msg = new PG_Label(this, PG_Rect(MARGIN_X, y, 
                           LINE_WIDTH, LINE_HEIGHT), 
                           s.c_str());
	    msg->SetAlignment(PG_TA_CENTER);
        d_msgs.push(msg);
        msgs.pop();
        y += (LINE_HEIGHT +2);
    }


    Stack *stack;
    getHeroById(quest->getHeroId(), &stack);
    debug("Stack of the hero: " << stack);

    Player *p = stack->getPlayer();
    debug("Owner of the hero: " << p->getName());

    // now the reward - at the moment a very simple one (just money):
	int cur_gold = p->getGold();

    debug("Current gold: " << cur_gold);

    p->giveReward(1, stack);
    //this also includes the gold-only hack
    int gold_added = p->getGold() - cur_gold;
    debug("Gold added: " << gold_added);

	snprintf(buffer, 100, ngettext("You have been rewarded with %i gold piece!",
                             "You have been rewarded, with %i gold pieces!", gold_added),
            gold_added);
	msg = new PG_Label(this, PG_Rect(MARGIN_X, y, LINE_WIDTH, LINE_HEIGHT), 
                       buffer);
	msg->SetAlignment(PG_TA_CENTER);
    d_msgs.push(msg);

	d_b_close = new PG_Button(this, 1, PG_Rect(MARGIN_X, my_height - 40, 
                              LINE_WIDTH, 30), 
                              _("Close"));
	d_b_close->SetEventObject(MSG_BUTTONCLICK, this, 
                     (MSG_CALLBACK_OBJ)&QuestCompletedDialog::b_closeClicked);
}

QuestCompletedDialog::~QuestCompletedDialog()
{
    delete d_b_close;
    while (d_msgs.empty() == false)
    {
        delete d_msgs.front();
        d_msgs.pop();
    }
}

PARAGUI_CALLBACK(QuestCompletedDialog::b_closeClicked)
{
	SendMessage(this, MSG_MODALQUIT, 0, 0);
	return true;
}

bool QuestCompletedDialog::eventKeyDown(const SDL_KeyboardEvent* key)
{
    switch(key->keysym.sym)
    {
        case SDLK_RETURN:
            b_closeClicked(0,0,0,0);
            break;
        default:
            break;
    }
    return true;
}

// End of file
