//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef RADDUNIT_H
#define RADDUNIT_H

#include <paragui.h>
#include <string>
#include <sigc++/sigc++.h>
#include "Reaction.h"
#include "stack.h"

/** Adds a specific army to a given player when triggered.
  * 
  * @note Don't reycle this reaction or the army it contains under any
  * circumstances unless you want to get into trouble. The army this reaction
  * has must have a unique id.
  * 
  * @note The position where the army is placed has to be free of other stacks.
  * If this cannot be guaranteed, be aware that the stack will then be placed
  * on the next tile where this is possible. If there are several tiles, no order
  * is guaranteed. So be careful if you place a new stack on a tiny island. :)
  *
  * TODO: This reaction currently tries to place the army in a separate stack at
  * the specified position or at one of the eight tiles around it. If they are
  * blocked, this reaction fails.
  * 
  * See Reaction for a more detailed explanation of reactions.
  */


class RAddUnit : public Reaction, public SigC::Object
{
    public:
        /** default constructor
          * 
          * @param army     the army to add (note: shallow copy! Don't use this army
          *                 any more)
          * @param player   the player who gets the army
          */
        RAddUnit(Stack* unit, unsigned int player, PG_Point pos);

        //! Loads the data from a savegame
        RAddUnit(XML_Helper* helper);
        ~RAddUnit();

        //! Saves the data to a savegame. See XML_Helper for more information
        bool save(XML_Helper* helper);

        //! Triggers the reaction
        bool trigger();

        //! Returns the specialized army
        Stack* getStack() {return d_stack;}
        
        //! Returns the player
        Player* getPlayer();

        //! Callback for loading, useless otherwise
        bool loadStack(std::string tag, XML_Helper* helper);
        
    protected:
        Stack* d_stack;
        unsigned int d_player;
        PG_Point d_pos;
};

#endif //RADDUNIT_H
