//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <algorithm>
#include "ai_fast.h"
#include "playerlist.h"
#include "armysetlist.h"
#include "FightDialog.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruinlist.h"
#include <fstream>
#include "path.h"
#include "GameMap.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
//#define debug(x)

AI_Fast::AI_Fast(string name, string armyset, SDL_Color color)
    :RealPlayer(name, armyset, color, Player::AI_FAST)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Fast::newLevelArmy));
}

AI_Fast::AI_Fast(const Player& player)
    :RealPlayer(player)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Fast::newLevelArmy));
}

AI_Fast::AI_Fast(XML_Helper* helper)
    :RealPlayer(helper)
{
    snewlevelAiArmy.connect(SigC::slot((*this), &AI_Fast::newLevelArmy));
}

AI_Fast::~AI_Fast()
{
}

bool AI_Fast::startTurn()
{
    debug(getName() << ": AI_Fast::start_turn")

    clearActionlist();

    //this is a recursively-programmed quite staightforward AI,we just call:
    computerTurn();

    d_stacklist->setActivestack(0);

    return true;
}

bool AI_Fast::invadeCity(City* c)
{
    //stupid ai player always occupies an enemy city
    bool retval = cityOccupy(c);
    sinvadingCity.emit(c);
    soccupyingCity.emit(c);

    return retval;
}

bool AI_Fast::recruitHero(Hero* hero, int cost)
{
    return true;    //always recruit heroes
}

void AI_Fast::computerTurn()
{
    // easy AI player, but very fast!
    for (Stacklist::iterator it = d_stacklist->begin(); it != d_stacklist->end(); ++it)
    {
        d_stacklist->setActivestack(*it);
        Stack* s = *it;
        if (s->size() >= 8)
        {
            City* city = Citylist::getInstance()->getNearestEnemyCity(s->getPos());
            Location *target = city;
            if (!target)    //all cities conquered?
                return;
            PG_Point p = target->getPos();
            s->getPath()->calculate(s, p);
            stackMove(s);
        }
        else
        {
            City *target = Citylist::getInstance()->getNearestFriendlyCity(s->getPos());
            if (!target) // no friendly city?
                return;
            if (target->contains(s->getPos()))
            {
                debug(getName() << ": We are already at " << target->getName())
                return;
            }
            else
            {
                PG_Point p = target->getPos();
                // FIXME - am I allowed to deallocate alt at the end of this code?
                // Or has something got a reference to it?
                PG_Point *alt = new PG_Point;
                debug(getName() << ": We have decided to go to " << target->getName())
                s->getPath()->calculate(s, p);
                if (stackMove(s))
                {
                    debug(getName() << ": The best position in the city is available")
                    return;
                }
                alt->x = p.x + 1;
                alt->y = p.y;
                s->getPath()->calculate(s, *alt);
                if (stackMove(s))
                {
                    debug(getName() << ": Moving into the northeast")
                    return;
                }
                alt->x = p.x;
                alt->y = p.y + 1;
                s->getPath()->calculate(s, *alt);
                if (stackMove(s))
                {
                    debug(getName() << ": Moving into the southwest")
                    return;
                }
                alt->x = p.x + 1;
                alt->y = p.y + 1;
                s->getPath()->calculate(s, *alt);
                if (stackMove(s))
                {
                    debug(getName() << ": Moving into the southeast")
                    return;
                }
            }
        }
        // a stack has died -> further iterating leads to a crash so we
        // start again (recursive programming, therefore the break
        // afterwards is _VERY_ important)
        if (!d_stacklist->getActivestack())
        {
            computerTurn();
            break;
        }
    }
}

void AI_Fast::newLevelArmy(Army* a)
{
    //advancing a level
    //Must write a random choose or a more complex one

    //a->setDefense((d_army->getDefense()+1));
    //a->setVitality((d_army->getVitality()+1));
    a->setOrigStrength((a->getOrigStrength()+1));
    //a->setMaxHp((d_army->getMaxHp()+(int)(d_army->getMaxHp()/5)));
    //a->setMaxMoves((d_army->getMaxMoves()+(int)(d_army->getMaxMoves()/5)));

    a->printAllDebugInfo();

    debug("AI NEWLEVEL -- value chosen=" << "STRENGTH")
}

// End of file
