//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef AI_FAST_H
#define AI_FAST_H

#include <string>
#include <list>
#include <pgmessageobject.h>
#include "defs.h"
#include "real_player.h"

/** Quite straightforward AI
  * 
  * This is a quite dumb AI player. It doesn't calculate much, that is why it is
  * called "fast". The basic algorithm is: Loop through all stacks. If a stack
  * has 8 units, look for the nearest enemy city and send the stack there. As I
  * said, dumb. It ignores enemy armies, and if a stack looses an army during a
  * fight it will just stand around.
  *
  * See the Player class for the derivation scheme
  */

class AI_Fast : public RealPlayer
{
    public:
        /** Default constructor
          * 
          * @param name         the name of the player
          * @param armyset      the armyset of the player
          * @param color        the player's color
          */
        AI_Fast(std::string name, std::string armyset, SDL_Color color);

        //! Copy constructor
        AI_Fast(const Player&);

        //! Loading constructor. See XML_Helper for an explanation.
        AI_Fast(XML_Helper* helper);
        ~AI_Fast();
        
        /** This function is called whenever the player's turn starts. As soon
          * as it returns, the player's turn ends.
          */
        bool startTurn();

        //! Callback when an ary of the player advances a level
        void newLevelArmy(Army* a);

        //! Callback when the player invades a city
        bool invadeCity(City* c);

        //! Callback when a hero offers his services
        bool recruitHero(Hero* hero, int cost);

    private:
        //! The actual core function of the ai's logic.
        void computerTurn(); 
};

#endif // AI_FAST_H
