//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "temple.h"
#include "counter.h"
#include "GameMap.h"
#include "QuestsManager.h"
#include "stack.h"

Temple::Temple(PG_Point pos, std::string name)
    :Location(name, pos)
{
    d_id = fl_counter->getNextId();
}

Temple::Temple(XML_Helper* helper)
    :Location()
{
    d_size = 1;

    helper->getData(d_id, "id");
    helper->getData(d_name, "name");

    int i;
    helper->getData(i, "x");    d_pos.x = i;
    helper->getData(i, "y");    d_pos.y = i;

    //mark the location on the game map as occupied by a temple
    GameMap::getInstance()->getTile(d_pos.x, d_pos.y)->setBuilding(Maptile::TEMPLE);
}

Temple::~Temple()
{
}

bool Temple::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("temple");
    retval &= helper->saveData("id", d_id);
    retval &= helper->saveData("name", d_name);
    retval &= helper->saveData("x", d_pos.x);
    retval &= helper->saveData("y", d_pos.y);
    retval &= helper->closeTag();
    
    return retval;
}

Quest* Temple::receiveQuest(Stack* s)
{
    culpritsType heroes;
    s->getHeroes(heroes);
    
    // Try to assign each hero in turn a quest. If it doesn't work, we assume,
    // he already has one.
    for (unsigned int i = 0; i < s->size(); i++)
    {
        Quest* q = QuestsManager::getInstance()->createNewQuest(heroes[i]);
        if (q)
            return q;
    }
    
    return 0;
}
