//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <assert.h>
#include <unistd.h>    //for usleep
#include <pgmessagebox.h>

#include "w_edit.h"
#include "RuinSearchDialog.h"
#include "CityOccupationDialog.h"
#include "FightDialog.h"
#include "bigmap.h"
#include "smallmap.h"
#include "army.h"
#include "hero.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruinlist.h"
#include "templelist.h"
#include "ruin.h"
#include "temple.h"
#include "GameMap.h"
#include "playerlist.h"
#include "stackinfo.h"
#include "hero_offer.h"
#include "path.h"
#include "Configuration.h"
#include "File.h"
#include "NextPlayerPopup.h"
#include "signals.h"
#include "QuestsManager.h"
#include "ArmyLevelDialog.h"
#include "ArmyMedalDialog.h"
#include "EPRound.h"
#include "RUpdate.h"
#include "tooltip.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
#define debug(x)

W_Edit::W_Edit(GameScenario* gameScenario, PG_Widget* parent, PG_Rect rect)
        :PG_Widget(parent, rect), d_gameScenario(gameScenario), d_lock(false)
{
    myrect=rect;

    // this defines how many squares the bigmap shows! this is important
    // for the smallmap, because the viewrect depends on it
    int x_squares = (rect.my_width - 219) / 50;
    int y_squares = (rect.my_height - 190) / 50;

    // load button picture
    d_pic_prev = File::getMiscPicture("button_prev.png", false);
    d_pic_next = File::getMiscPicture("button_next.png", false);
    d_pic_defend = File::getMiscPicture("button_guard.png", false);
    d_pic_move = File::getMiscPicture("button_move.png", false);
    d_pic_moveall = File::getMiscPicture("button_moveall.png", false);
    d_pic_nextwithmove = File::getMiscPicture("button_nextwmove.png", false);
    d_pic_defendAndNext = File::getMiscPicture("button_guardnext.png", false);
    d_pic_search = File::getMiscPicture("button_search.png", false);
    d_pic_nextTurn = File::getMiscPicture("button_endturn.png", false);
    d_pic_centerCurrent = File::getMiscPicture("button_center.png", false);

    //some graphical finetuning: place the smallmap in the middle of the piece
    //above the buttons (in the middle of x = (width - 109) -- (width -7) and
    //y = 20 -- 122
    PG_Rect smallrect;
    smallrect.x = rect.my_width - 58 - (GameMap::getSize()/2);
    smallrect.y = 71 - (GameMap::getSize()/2);
    smallrect.w = GameMap::getSize() + 2;
    smallrect.h = smallrect.w;
    d_smallmap = new SmallMap(this, smallrect, x_squares, y_squares);

    // Create the bigmap...
    d_bigmap = new BigMap(this, 
            PG_Rect(60, 60, x_squares*50, y_squares*50));
    d_bigmap->setViewrect(d_smallmap->getViewrect());
    
    // ...and the borders around. The borders consist of 8 scrolling buttons and
    // 8 connecting pieces. We start with the top left and go clockwise.
    // Regard the placing and the sizes as semi-magical and ignore them unless you want
    // to actually change something here.
    d_b_scroll[0] = new PG_Button(this, 10, PG_Rect(20, 20, 40, 40));
    d_b_scroll[1] = new PG_Button(this, 11, PG_Rect(40 + x_squares*25, 20, 40, 40));
    d_b_scroll[2] = new PG_Button(this, 12, PG_Rect(60 + x_squares*50, 20, 40, 40));
    d_b_scroll[3] = new PG_Button(this, 13, PG_Rect(60 + x_squares*50, 40 + y_squares*25, 40, 40));
    d_b_scroll[4] = new PG_Button(this, 14, PG_Rect(60 + x_squares*50, 60 + y_squares*50, 40, 40));
    d_b_scroll[5] = new PG_Button(this, 15, PG_Rect(40 + x_squares*25, 60 + y_squares*50, 40, 40));
    d_b_scroll[6] = new PG_Button(this, 16, PG_Rect(20, 60 + y_squares*50, 40, 40));
    d_b_scroll[7] = new PG_Button(this, 17, PG_Rect(20, 40 + y_squares*25, 40, 40));

    d_border[0] = new PG_ThemeWidget(this, PG_Rect(60, 25, x_squares*25 - 20, 35), true);
    d_border[1] = new PG_ThemeWidget(this, PG_Rect(80 + x_squares*25, 25, x_squares*25 - 20, 35), true);
    d_border[2] = new PG_ThemeWidget(this, PG_Rect(60 + x_squares*50, 60, 35, y_squares*25 - 20), true);
    d_border[3] = new PG_ThemeWidget(this, PG_Rect(60 + x_squares*50, 80 + y_squares*25, 35, y_squares*25 - 20), true);
    d_border[4] = new PG_ThemeWidget(this, PG_Rect(80 + x_squares*25, 60 + y_squares*50, x_squares*25 - 20, 35), true);
    d_border[5] = new PG_ThemeWidget(this, PG_Rect(60, 60 + y_squares*50, x_squares*25 - 20, 35), true);
    d_border[6] = new PG_ThemeWidget(this, PG_Rect(25, 80 + y_squares*25, 35, y_squares*25 - 20), true);
    d_border[7] = new PG_ThemeWidget(this, PG_Rect(25, 60, 35, y_squares*25 - 20), true);

    // Load the images
    for (int i = 0; i < 8; i++)
    {
        char buffer[80];
        buffer[79] = '\0';

        snprintf(buffer, 79, "scroll%i", i);
        d_scrollsurf[i] = File::getBorderPic(std::string(buffer));

        snprintf(buffer, 79, "scroll%i_on", i);
        d_scrollsurfon[i] = File::getBorderPic(std::string(buffer));

        if (i % 2 == 0)
        {
            snprintf(buffer, 79, "border%i", i/2);
            d_bordersurf[i/2] = File::getBorderPic(std::string(buffer));
        }
    }

    // And finish the borders
    for (int i = 0; i < 8; i++)
    {
        d_b_scroll[i]->SetIcon(d_scrollsurf[i], 0, d_scrollsurfon[i]);
        d_b_scroll[i]->SetBorderSize(0, 0, 0);
        d_b_scroll[i]->SetShift(0);
        d_b_scroll[i]->SetTransparency(255, 255, 255);
        d_b_scroll[i]->SetEventObject(MSG_BUTTONCLICK, this,
                        (MSG_CALLBACK_OBJ)&W_Edit::b_scrollClicked);

        d_border[i]->SetBackground(d_bordersurf[i/2]);
        d_border[i]->SetBackgroundBlend(0);
    }
   
    // And do the rest of the setup

    d_stackinfo = new Stackinfo(this, 
            PG_Rect(20, rect.my_height - 80, 360, 72));
    d_nextPlayerPopup = new NextPlayerPopup(this, 
            PG_Rect((my_width-300)/2, (my_height-200)/2, 300, 200));
    d_nextPlayerPopup->Hide();

    l_turns = new PG_Label(this,
            PG_Rect(rect.my_width - 134, 130, 100, 20), "");
    l_gold = new PG_Label(this,
            PG_Rect(rect.my_width - 134, 150, 100, 20), "");

    b_prev = new PG_Button(this, 1,
            PG_Rect(rect.my_width - 134, 170, 40, 40));
    b_next = new PG_Button(this, 2,
            PG_Rect(rect.my_width - 94, 170, 40, 40));
    b_nextwithmove = new PG_Button(this, 2,
            PG_Rect(rect.my_width - 54, 170, 40, 40));
    b_move = new PG_Button(this, 3,
            PG_Rect(rect.my_width - 134, 210, 40, 40));
    b_moveall = new PG_Button(this, 9, 
            PG_Rect(rect.my_width - 94, 210, 40, 40));
    b_centerCurrent = new PG_Button(this, 4,
            PG_Rect(rect.my_width - 54, 210, 40, 40));
    b_defend = new PG_Button(this, 5,
            PG_Rect(rect.my_width - 134, 250, 40, 40));
    b_defendAndNext = new PG_Button(this, 6,
            PG_Rect(rect.my_width - 94, 250, 40, 40));
    b_search = new PG_Button(this, 7,
            PG_Rect(rect.my_width - 134, 290, 40, 40));
    b_nextTurn = new PG_Button(this, 8,
            PG_Rect(rect.my_width - 94, 290, 40, 40));


    sdyingStack.connect(SigC::slot((*this), &W_Edit::stackDied));
    snewlevelArmy.connect(SigC::slot((*this), &W_Edit::newLevelArmy));
    snewmedalArmy.connect(SigC::slot((*this), &W_Edit::newMedalArmy));

    // this has to be done for each player!!!
    for (Playerlist::iterator it = Playerlist::getInstance()->begin();
            it != Playerlist::getInstance()->end(); ++it)
    {
        (*it)->schangingStatus.connect(SigC::slot((*this), &W_Edit::updateStatus));
        (*it)->supdatingStack.connect(SigC::slot((*this), &W_Edit::stackUpdate));

        (*it)->sinvadingCity.connect(SigC::slot((*this), &W_Edit::cityOccupied));
        (*it)->sinterruptTimers.connect(SigC::slot((*this), &W_Edit::stopTimers));
        (*it)->scontinueTimers.connect(SigC::slot((*this), &W_Edit::startTimers));

        //We can connect to all players, since only the human players will raise
        //the signal.
        (*it)->srecruitingHero.connect(SigC::slot((*this), &W_Edit::heroJoins));
    }

    d_smallmap->schangingViewrect.connect(SigC::slot((*this), &W_Edit::bigmapRedraw));
    d_bigmap->schangingViewrect.connect(SigC::slot((*this), &W_Edit::smallmapRedraw));
    d_bigmap->sselectingStack.connect(SigC::slot((*this), &W_Edit::bigmapStackSelected));
    
    b_move->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_moveClicked);
    b_moveall->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_moveAllClicked);
    b_prev->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_prevClicked);
    b_nextwithmove->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_nextwithmoveClicked);
    b_next->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_nextClicked);
    b_defend->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_defendClicked);
    b_defendAndNext->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_defendAndNextClicked);
    b_search->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_searchClicked);
    b_nextTurn->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&W_Edit::b_nextTurnClicked);
    b_centerCurrent->SetEventObject(MSG_BUTTONCLICK, this,
                     (MSG_CALLBACK_OBJ)&W_Edit::b_centerCurrentClicked);

    b_nextTurn->SetIcon(d_pic_nextTurn, 0, 0);
    b_move->SetIcon(d_pic_move, 0, 0);
    b_moveall->SetIcon(d_pic_moveall, 0, 0);
    b_prev->SetIcon(d_pic_prev, 0, 0);
    b_next->SetIcon(d_pic_next, 0, 0);
    b_nextwithmove->SetIcon(d_pic_nextwithmove, 0, 0);
    b_defend->SetIcon(d_pic_defend, 0, 0);
    b_defendAndNext->SetIcon(d_pic_defendAndNext, 0, 0);
    b_search->SetIcon(d_pic_search, 0, 0);
    b_centerCurrent->SetIcon(d_pic_centerCurrent, 0, 0);

    //set up a NextTurn object
    d_nextTurn = new NextTurn(d_gameScenario->getTurnmode());
    d_nextTurn->splayerStart.connect(SigC::slot((*this), &W_Edit::checkPlayer));
    d_nextTurn->snextRound.connect(SigC::slot((*d_gameScenario), &GameScenario::nextTurn));
            
    connectEvents();
    
    d_allDefending = false;
    checkButtons();
}

W_Edit::~W_Edit()
{
    //smallmap as well as bigmap have timers running, so first stop the timers
    //and then wait some microseconds to make sure we don't get problems with
    //the timers
    stopTimers();
    usleep(100);

    delete d_smallmap;
    delete d_bigmap;

    delete d_gameScenario;
    delete d_nextTurn;
    
    delete d_stackinfo;
    delete l_turns;
    delete l_gold;
    SDL_FreeSurface(d_pic_prev);
    SDL_FreeSurface(d_pic_next);
    SDL_FreeSurface(d_pic_defend);
    SDL_FreeSurface(d_pic_move);
    SDL_FreeSurface(d_pic_moveall);
    SDL_FreeSurface(d_pic_defendAndNext);
    SDL_FreeSurface(d_pic_search);
    SDL_FreeSurface(d_pic_nextTurn);
    SDL_FreeSurface(d_pic_centerCurrent);
    SDL_FreeSurface(d_pic_nextwithmove);
    delete b_prev;
    delete b_next;
    delete b_nextwithmove;
    delete b_move;
    delete b_moveall;
    delete b_defend;
    delete b_defendAndNext;
    delete b_search;
    delete b_nextTurn;
    delete b_centerCurrent;

    for (int i = 0; i < 8; i++)
    {
        delete d_b_scroll[i];
        delete d_border[i];
    }

    for (int i = 0; i < 8; i++)
    {
        SDL_FreeSurface(d_scrollsurf[i]);
        SDL_FreeSurface(d_scrollsurfon[i]);
    }
    for (int i = 0; i < 4; i++)
        SDL_FreeSurface(d_bordersurf[i]);
}

void W_Edit::connectEvents()
{
    std::list<Event*> elist = d_gameScenario->getEventlist();
    ERound* eround;
    EPRound* epround;

    for (std::list<Event*>::iterator it = elist.begin(); it != elist.end(); it++)
    {
        Reaction* r = (*it)->getReaction(0);
        int i = 1;
        while (r != 0)
        {
            if (r->getType() == Reaction::UPDATE)
            {
                RUpdate* reac = dynamic_cast<RUpdate*>(r);
                reac->supdating.connect(SigC::slot(*d_bigmap, &BigMap::Redraw));
            }
            r = (*it)->getReaction(i);
            i++;
        }
                    
        
        switch((*it)->getType())
        {
            //the round event needs rather much help with its signals
            case Event::ROUND:
                eround = dynamic_cast<ERound*>(*it);
                eround->sgettingRound.connect(SigC::slot(*d_gameScenario,
                                                        &GameScenario::getTurns));
                d_nextTurn->snextRound.connect(SigC::slot(*eround, &ERound::trigger));
                break;

            case Event::PROUND:
                epround = dynamic_cast<EPRound*>(*it);
                epround->sgettingRound.connect(SigC::slot(*d_gameScenario,
                                                        &GameScenario::getTurns));
                d_nextTurn->snextTurn.connect(SigC::slot(*epround, &EPRound::trigger));
                break;
                
            default:
                continue;
        }
    }
}

PARAGUI_CALLBACK(W_Edit::b_scrollClicked)
{
    static int dx[] = {-1, 0, 1, 1, 1, 0, -1, -1};
    static int dy[] = {-1, -1, -1, 0, 1, 1, 1, 0};

    int pos = -1;
    for (int i = 0; i < 8; i++)
        if (widget == d_b_scroll[i])
        {
            pos = i;
            break;
        }

    if (pos < 0)
        return false;

    // Why not make things simple?
    helpSmallmap(dx[pos], dy[pos]);
    
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_nextTurnClicked)
{
    //clean up and hand over control to NextTurn class
    d_bigmap->stackDeselected();
    d_stackinfo->Hide();
    lockScreen();

    d_nextTurn->endTurn();
    return true;
}

void W_Edit::nextTurn()
{
    debug("nextTurn()");

    //set up everything for the next human player's turn

    if (!Playerlist::getActiveplayer())
    {
        return;
    }

    unlockScreen();
    
    d_allDefending = false;
    displayFirstCity();
    updateStatus();
    
    // human player
    if (Configuration::s_showNextPlayer)
    {
        pictureNextPlayer();
    }
}

// Print new status
void W_Edit::updateStatus()
{
    char buffer[80];
    sprintf(buffer, _("Turns: %i"), d_gameScenario->getTurns());
    l_turns->SetText(buffer);

    sprintf(buffer, _("Gold: %i"), Playerlist::getActiveplayer()->getGold());
    l_gold->SetText(buffer);
}

// find first city of activeplayer and view it if outside screen
void W_Edit::displayFirstCity()
{
    debug("displayFirstCity()");

    for (Citylist::iterator it = Citylist::getInstance()->begin();
        it != Citylist::getInstance()->end(); it++)
    {
        if ((*it).getPlayer() == Playerlist::getInstance()->getActiveplayer())
        {
            d_bigmap->centerView((*it).getPos());
            break;
        }
    }
}

// pop up message window for next player
void W_Edit::pictureNextPlayer()
{
    debug("pictureNextPlayer()");

    d_nextPlayerPopup->setName(Playerlist::getInstance()->getActiveplayer()->getName());
    d_nextPlayerPopup->Show();
    SetCapture();
}

bool W_Edit::heroJoins(Hero* hero, int gold)
{
    debug("heroJoins()");

    bool resetpopup = false;
    //fix for a bug where NextPlayerPopup and Hero_offer would clash
    //because of a set capture for all mouse events
    if (d_nextPlayerPopup->IsVisible())
    {
        d_nextPlayerPopup->Hide();
        ReleaseCapture();
        resetpopup = true;
    }

    char buffer[101];
    buffer[100]='\0';
    snprintf(buffer, 100, _("Hero offer for %s"),
            Playerlist::getActiveplayer()->getName().c_str());

    Hero_offer hero_offer(this, hero, gold);
    hero_offer.SetTitle(buffer);
    hero_offer.Show();
    hero_offer.RunModal();
    hero_offer.Hide();

    if (resetpopup)
    {
        d_nextPlayerPopup->Show();
        SetCapture();
    }

    return hero_offer.getRetval();
}

PARAGUI_CALLBACK(W_Edit::b_prevClicked)
{
    Stack* stack = Playerlist::getActiveplayer()->getStacklist()->setPrev();
    debug("Setting active stack = " << stack)
    if (stack) d_bigmap->stackSelected();
    else d_allDefending = true;

    checkButtons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_nextClicked)
{
    Stack* stack = Playerlist::getActiveplayer()->getStacklist()->setNext();
    debug("Setting active stack = " << stack)
    if (stack) d_bigmap->stackSelected();
    else d_allDefending = true;

    checkButtons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_nextwithmoveClicked)
{
    Stack* stack = Playerlist::getActiveplayer()->getStacklist()->setNextWithMove();
    debug("Setting active stack = " << stack)
    if (stack) d_bigmap->stackSelected();
    else d_allDefending = true;

    checkButtons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_moveClicked)
{
    Playerlist::getActiveplayer()->stackMove(Playerlist::getActiveplayer()->getActivestack());
    checkButtons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_moveAllClicked)
{
    vector<unsigned int> movedid;
    vector<unsigned int>::iterator it;
    bool alreadymoved=false;
    Stack* stack =0;

    while ((stack = Playerlist::getActiveplayer()->getStacklist()->setNextWithMove())) 
    {

        debug(" newid=-->"<< stack->getId())
	sleep(2);  

        if(stack->getPath()->size() > 0) 
        {
            debug("Path>0 and movedid-->"<< stack->getId())
            d_bigmap->stackSelected();
            Playerlist::getActiveplayer()->stackMove(Playerlist::getActiveplayer()->getActivestack());
            movedid.push_back(stack->getId());
        }	
        else
	{
   	    debug("path=0 we check if this unit has already moved and we are in loop"); 
	    it=movedid.begin();
	    while(it!=movedid.end()) 
	    {
	        if (*it==stack->getId()) 
	        {
	            alreadymoved=true;
	            break;
	        }
	        it++;
 	    }
	}

        if (alreadymoved)
	  break;
    }
    checkButtons();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_defendClicked)
{
    Playerlist::getActiveplayer()->getActivestack()->setDefending(true);
    d_bigmap->stackDeselected();
    d_stackinfo->Hide();
    checkButtons();

    return true;
}

PARAGUI_CALLBACK(W_Edit::b_defendAndNextClicked)
{
    //store the stack for later
    Stack* temp = Playerlist::getActiveplayer()->getActivestack();

    // defendAndNext means setting the stack to defending...
    b_defendClicked(GetID(), this, 1, 0);

    // ...and getting the next stack after this stack
    Playerlist::getActiveplayer()->getStacklist()->setActivestack(temp);
    b_nextClicked(GetID(), this, 1, 0);
    
    // If there are no other stacks, unset the active stack
    if (Playerlist::getActiveplayer()->getActivestack() == temp)
        b_defendClicked(GetID(), this, 1, 0);

    return true;
}

PARAGUI_CALLBACK(W_Edit::b_centerCurrentClicked)
{
    Stack *stack = Playerlist::getActiveplayer()->getActivestack();
    if (stack) 
        d_bigmap->stackSelected();
    return true;
}

PARAGUI_CALLBACK(W_Edit::b_searchClicked)
{
    Stack* stack = Playerlist::getActiveplayer()->getActivestack();

    Player* p = Playerlist::getInstance()->getActiveplayer();
    Ruin* ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
    Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());

    debug("search: ruin = " << ruin << ", temple = " << temple)

    if (ruin)
    {
        int cur_gold = p->getGold();

        if (!(p->stackSearchRuin(stack, ruin)))
        {
            if (stack->empty())
            {
                stackDied(stack);
                Stacklist::deleteStack(stack);
                return true;
            }
            stackRedraw(GetID(), this, 1, 0);
            return true;
        }

        stackRedraw(GetID(), this, 1, 0);

        //this also includes the gold-only hack
        int gold_added = p->getGold() - cur_gold;

        RuinSearchDialog dialog(this, gold_added);
        dialog.Show();
        dialog.RunModal();
        dialog.Hide();

        updateStatus();
    }
    
    else if ((temple) && (temple->search()))
    {

        const unsigned int w1 = 230;
        const unsigned int h1 = 110;
        PG_MessageBox mb(this, PG_Rect((my_width-w1)/2, 
                                       (my_height-h1)/2, w1, h1),
                _("Temple"), _("Select an action"),
                PG_Rect(10, h1-40, 100, 30), _("Bless"),
                PG_Rect(120, h1-40, 100, 30), _("Quest"),
                PG_TA_CENTER
                );
        mb.Show();
        int decision = mb.WaitForClick();
        debug("decision = " << decision);
        mb.Hide();
        if (decision == 1)
        {
            unsigned int w1 = 240;
            unsigned int h1 = 110;
            PG_MessageBox mb(this, PG_Rect((my_width-w1)/2, 
                                           (my_height-h1)/2, w1, h1),
                    _("Temple"), _("Your armies have been blessed"),
                    PG_Rect(80, 70, 80, 30), _("OK"));
            mb.Show();
            mb.WaitForClick();
            mb.Hide();
            p->stackVisitTemple(stack, temple);
        }
        else
        {
            Quest *q = temple->receiveQuest(stack);
            Hero* hero = dynamic_cast<Hero*>(stack->getFirstHero());

            if (q)
                for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
                    if ((*it)->getId() == q->getHeroId())
                        hero = dynamic_cast<Hero*>(*it);
            
            std::string title = _("Quest - ") + hero->getName();
            const char *txt = q ? q->getDescription()
                                : _("This hero already has a quest");
            PG_MessageBox mb(this, PG_Rect((my_width-400)/2, (my_height-160)/2, 
                             400, 160), title.c_str(), txt,
            PG_Rect((400 - 80)/2, 160 - 30 - 10, 80, 30), _("OK"));
            mb.Show();
            mb.WaitForClick();
            mb.Hide();
        }
    }

    return true;
}

// the parameter is currently not used (=0), but may be used for more detailed
// descriptions later on
void W_Edit::stackUpdate(Stack* s)
{
    debug("W_Edit: stackUpdate()");

    d_bigmap->stackMoved();
    d_smallmap->Redraw();

    d_stackinfo->readData();

    checkButtons();
}

// s is currently unused, but can later be filled with reasonable data
void W_Edit::stackDied(Stack* s)
{
    debug("stackDied()");
    d_bigmap->stackDeselected();
    d_smallmap->Redraw();
    d_stackinfo->Hide();
    checkButtons();
}


void W_Edit::newLevelArmy(Army* a)
{
    debug("NEWLEVEL Dialog SHOW")

    // Don't show this dialog if computer armies advance.
    if (Playerlist::getActiveplayer()->getType() != Player::HUMAN)
        return;

    ArmyLevelDialog dialog(a, 0, PG_Rect(200, 100, 430, 230));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    // Here Which functions should i add to refresh?? (Andrea)
    // Do we need them (as checkButtons() or others)?
    // UL: only need to redraw the stackinfo
    d_stackinfo->Redraw();
}

void W_Edit::newMedalArmy(Army* a)
{
    // We don't want to have medal awards of computer players displayed
    if (Playerlist::getActiveplayer()->getType() != Player::HUMAN)
        return;
    
    debug("NEWMedal Dialog SHOW")
    std::cerr << "NEWMedal Dialog SHOW" << std::endl;
    ArmyMedalDialog dialog(a, 0, PG_Rect(200, 100, 430, 230));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();
 
    // Here Which functions should i add to refresh?? (Andrea)
    // Do we need them (as checkButtons() or others)?
    // UL: only need to redraw the stackinfo
    d_stackinfo->Redraw();
}

PARAGUI_CALLBACK(W_Edit::stackRedraw)
{
    d_bigmap->Redraw();
    d_stackinfo->readData();
    checkButtons();
    return true;
}

void W_Edit::bigmapStackSelected(Stack* s)
{
    d_stackinfo->Show();
    d_stackinfo->readData();
    checkButtons();
}

void W_Edit::cityOccupied(City* city)
{
    debug("cityOccupied()");

    // if a computer makes it's turn and occupied a city, we shouldn't
    // show a modal dialog :)

    if (!d_lock)
    {
        CityOccupationDialog dialog(city);
        dialog.Show();
        dialog.RunModal();
        dialog.Hide();
    }
   
    Playerlist::getInstance()->checkPlayers();
    d_bigmap->Redraw();
    d_stackinfo->readData();
    checkButtons();
}

void W_Edit::bigmapRedraw()
{
    d_bigmap->Redraw();
}

void W_Edit::smallmapRedraw()
{
    d_smallmap->Redraw();
}

void W_Edit::lockScreen()
{
    debug("lockScreen()");

    //don't accept modifying user input from now on
    d_bigmap->setEnable(false);
    b_nextTurn->EnableReceiver(false);
    b_prev->EnableReceiver(false);
    b_next->EnableReceiver(false);
    b_nextwithmove->EnableReceiver(false);
    b_defend->EnableReceiver(false);
    b_move->EnableReceiver(false);
    b_moveall->EnableReceiver(false);
    b_search->EnableReceiver(false);
    b_defendAndNext->EnableReceiver(false);
    b_centerCurrent->EnableReceiver(false);
    d_lock = true;
    stopTimers();   //on computer turns, strange things may happen if the timers
                    //are still active
}

//the inverse of lock_screen()

void W_Edit::unlockScreen()
{
    debug("unlockScreen()");
    b_nextTurn->EnableReceiver(true);
    d_bigmap->setEnable(true);
    d_lock = false;
    checkButtons();
    startTimers();
}

void W_Edit::stopTimers()
{
    d_bigmap->interruptTimer();
    d_smallmap->interruptTimer();
}

void W_Edit::startTimers()
{
    //never start timers with the computer player's turn
    if (d_lock)
        return;

    d_bigmap->restartTimer();
    d_smallmap->restartTimer();
}

void W_Edit::checkButtons()
{
    if (d_lock) return;
    
    debug("checkButtons()");

    // first assume that all buttons are disabled
    bool move = false;
    bool moveall = false;
    bool defend = false;
    bool defendAndNext = false;
    bool search = false;
    bool prev = false;
    bool next = false;
    bool nextwmove = false;
    bool center = false;

    // now check which buttons should be enabled
    if (!d_allDefending) 
    {
        next = true;
        prev = true;
        nextwmove = true;
    }

    Stack* stack = Playerlist::getActiveplayer()->getActivestack();
    if (stack)
    {
        defend = true;
        center = true;

        if (stack->getPath()->size() > 0 && stack->getPlayer()->enoughMoves(stack))
        {
            move = true;
            moveall = true;
        }
        //TBD what if there is only 1 army and it isn't defending?
        // should be something like (!all defend) && (further_stacks)
        // Answer: Then the army defends and no stack is selected. This
        // behaviour sounds OK to me...
        if (!d_allDefending)
        {
            defendAndNext = true;
        }

        if (stack->hasHero())
        {
            Ruin* ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
            Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());
            if ((ruin && !ruin->isSearched()) || temple)
            {
                search = true;
            }
        }
    }

    // now set the button states according to the calculations


    b_move->EnableReceiver(move);
    b_moveall->EnableReceiver(moveall);
    b_defend->EnableReceiver(defend);
    b_defendAndNext->EnableReceiver(defendAndNext);
    b_search->EnableReceiver(search);
    b_prev->EnableReceiver(prev);
    b_next->EnableReceiver(next);
    b_nextwithmove->EnableReceiver(nextwmove);
    b_centerCurrent->EnableReceiver(center);

    int x;
    prev ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_prev, SDL_SRCALPHA, x);
    next ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_next, SDL_SRCALPHA, x);
    nextwmove ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_nextwithmove, SDL_SRCALPHA, x);
    move ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_move, SDL_SRCALPHA, x);
    moveall ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_moveall, SDL_SRCALPHA, x);
    defend ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_defend, SDL_SRCALPHA, x);
    defendAndNext ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_defendAndNext, SDL_SRCALPHA, x);
    search ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_search, SDL_SRCALPHA, x);
    center ? x = 255 : x = 80;
    SDL_SetAlpha(d_pic_centerCurrent, SDL_SRCALPHA, x);

    // redraw the buttons (SetIcon doesn't do it)
    // don't redraw everything, or bigmap is drawn again
    b_move->Redraw();
    b_move->Update();
    b_moveall->Redraw();
    b_moveall->Update();
    b_defend->Redraw();
    b_defendAndNext->Redraw();
    b_search->Redraw();
    b_prev->Redraw();
    b_next->Redraw();
    b_nextwithmove->Redraw();
    b_centerCurrent->Redraw();
}

void W_Edit::startGame()
{
    debug ("start_game()");
    lockScreen();

    d_nextTurn->start();
}

void W_Edit::loadGame()
{
    if (Playerlist::getActiveplayer()->getType() == Player::HUMAN)
    {
        //human players want access to the controls and an info box
        unlockScreen();
        d_bigmap->stackDeselected();
        d_stackinfo->Hide();
        displayFirstCity();
        updateStatus();

        string s = _("Next Player: ");
        s += Playerlist::getInstance()->getActiveplayer()->getName();
        PG_MessageBox mb(this, PG_Rect(x, y, 200, 110), _("Next Player"),
                s.c_str(), PG_Rect(60, 70, 80, 30), _("OK"));
    }       
    else
    {
        lockScreen();
        updateStatus();
        Playerlist::getActiveplayer()->startTurn();
        d_nextTurn->endTurn();
    }
}

void W_Edit::stopGame()
{
    stopTimers();
    d_nextTurn->stop();
    d_gameScenario->deactivateEvents();
}

bool W_Edit::eventMouseButtonDown(const SDL_MouseButtonEvent* event)
{
    // only used to hide the "pictureNextPlayer" Box
    if(event->button == SDL_BUTTON_LEFT)
    {
        debug("eventMouseButtonDown: left button pressed");
        if (d_nextPlayerPopup->IsVisible())
        {
            d_nextPlayerPopup->Hide();
            ReleaseCapture();

            d_bigmap->Redraw();

            d_nextTurn->snextTurn.emit(Playerlist::getActiveplayer());
        }
    }
    return true;
}

bool W_Edit::checkPlayer(Player* p)
{
    //This function checks at the beginning of a new player's turn if
    //we want to interrupt the NextTurn cycle and give control back to
    //the graphics interface.
    //TBD: Currently this function only checks for a human player. You
    //can also have it check for e.g. escape key pressed to interrupt
    //an AI-only game to save/quit.

    if (p->getType() == Player::HUMAN)
    {
        //we have a human player, set up the screen and the player and return
        //false
        nextTurn();
        p->startTurn();
        return false;
    }

    //in any case, center on the active player's city and wait a moment
    displayFirstCity();
    usleep(250000);
    
    return true;
}

// This function is called by MainWindow when the user presses a cursor key.
void W_Edit::helpSmallmap(int arrowx ,int arrowy)
{
	d_smallmap->inputFunction(arrowx, arrowy);
}


void W_Edit::unselectStack()
{
    d_bigmap->stackDeselected();
    d_stackinfo->Hide();
    checkButtons();

    return;
}

bool W_Edit::eventMouseMotion(const SDL_MouseMotionEvent *event)
{
    // tooltips are causing severe problems (a strange segfault together with a
    // stack corruption), so we disable them here.
    return true;

    
    if (event->state == SDL_PRESSED)
        return true;
    
    if ( (event->x > (myrect.my_width - 134)) && (event->x <(myrect.my_width - 94) ) && (event->y > 195) && (event->y < 235) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Select the previous stack"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 94)) && (event->x <(myrect.my_width - 54) ) && (event->y > 195) && (event->y < 235) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Select the next stack"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 54)) && (event->x <(myrect.my_width - 14) ) && (event->y > 195) && (event->y < 235) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Select the next stack that can move"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 134)) && (event->x <(myrect.my_width - 94) ) && (event->y > 235) && (event->y < 275) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Let the selected stack move to destination"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 94)) && (event->x <(myrect.my_width - 54) ) && (event->y > 235) && (event->y < 275) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Let all stacks move to their destinations"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 54)) && (event->x <(myrect.my_width - 14) ) && (event->y > 235) && (event->y < 275) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Center window on selected stack"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 134)) && (event->x <(myrect.my_width - 94) ) && (event->y > 275) && (event->y < 305) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Let the stack defend"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 94)) && (event->x <(myrect.my_width - 54) ) && (event->y > 275) && (event->y < 305) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Let the stack defend and select the next stack"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 134)) && (event->x <(myrect.my_width - 94) ) && (event->y > 305) && (event->y < 345) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Search a ruin or a temple"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }
    if ( (event->x > (myrect.my_width - 94)) && (event->x <(myrect.my_width - 54) ) && (event->y > 305) && (event->y < 345) ) {
      ToolTip *tp=new ToolTip((PG_Widget*)this,_("Next turn"),event->x,event->y,myrect.my_width,myrect.my_height);
      tp=0; /* this is here just to avoid the warning, the tooltip is destroyed with mouse movement or button click*/
      return true;
    }

    return true;
}
// End of file

